//! @file Discret.hh
//! @brief 離散化クラス(テンプレート版)
//! @date 2019/07/19
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the BSD License.
// For details, see the License.txt file.

#ifndef DISCRET
#define DISCRET

#include <string>
#include <tuple>
#include <cassert>
#include "Matrix.hh"

namespace ARCS {	// ARCS名前空間
	//! @brief 離散化クラス(テンプレート版)
	class Discret {
		public:
			//! @brief 連続系状態方程式のA，B行列を離散化する関数(タプル版)
			//! @param[in]	Ac	連続系のA行列
			//! @param[in]	Bc	連続系のB行列
			//! @param[in]	Ts	サンプリング時間
			//! @return	Ad,Bd	離散系のA行列，離散系のB行列
			template <unsigned int NB, unsigned int MB>
			static std::tuple<Matrix<MB,MB>,Matrix<NB,MB>> GetDiscSystem(
				const Matrix<MB,MB>& Ac, const Matrix<NB,MB>& Bc, const double Ts
			){
				constexpr unsigned int Npade = 13;		// パデ近似の次数
				constexpr unsigned long Nint = 10000;	// 定積分の分割数
				return GetDiscSystem(Ac,Bc,Ts,Npade,Nint);	// 離散化
			}
			
			//! @brief 連続系状態方程式のA，B行列を離散化する関数 (タプル版，パデ近似と定積分分割数を自分で指定する場合)
			//! @param[in]	Ac	連続系のA行列
			//! @param[in]	Bc	連続系のB行列
			//! @param[in]	Ts	サンプリング時間
			//! @param[in]	Npade	パデ近似の次数
			//! @param[in]	Nint	定積分の分割数
			//! @return	Ad,Bd	離散系のA行列，離散系のB行列
			template <unsigned int NB, unsigned int MB>
			static std::tuple<Matrix<MB,MB>,Matrix<NB,MB>> GetDiscSystem(
				const Matrix<MB,MB>& Ac, const Matrix<NB,MB>& Bc,
				const double Ts, const unsigned int Npade, const unsigned long Nint
			){
				Matrix<MB,MB> Ad = expm(Ac*Ts,Npade);					// A行列の離散化
				Matrix<NB,MB> Bd = integral_expm(Ac,Ts,Nint,Npade)*Bc;	// B行列の離散化
				return std::forward_as_tuple(Ad,Bd);
			}
			
			//! @brief 連続系状態方程式のA，B行列を離散化する関数
			//! @param[in]	Ac	連続系のA行列
			//! @param[in]	Bc	連続系のB行列
			//! @param[in]	Ts	サンプリング時間
			//! @param[out]	Ad	離散系のA行列
			//! @param[out]	Bd	離散系のB行列
			template <unsigned int NB, unsigned int MB>
			static void GetDiscSystem(
				const Matrix<MB,MB>& Ac, const Matrix<NB,MB>& Bc,
				Matrix<MB,MB>& Ad, Matrix<NB,MB>& Bd, const double Ts
			){
				constexpr unsigned int Npade = 13;			// パデ近似の次数
				constexpr unsigned long Nint = 10000;		// 定積分の分割数
				GetDiscSystem(Ac,Bc,Ad,Bd,Ts,Npade,Nint);	// 離散化
			}
			
			//! @brief 連続系状態方程式のA，B行列を離散化する関数 (パデ近似と定積分分割数を自分で指定する場合)
			//! @param[in]	Ac	連続系のA行列
			//! @param[in]	Bc	連続系のB行列
			//! @param[out]	Ad	離散系のA行列
			//! @param[out]	Bd	離散系のB行列
			//! @param[in]	Ts	サンプリング時間
			//! @param[in]	Npade	パデ近似の次数
			//! @param[in]	Nint	定積分の分割数
			//! @return	Ad,Bd	離散系のA行列，離散系のB行列
			template <unsigned int NB, unsigned int MB>
			static void GetDiscSystem(
				const Matrix<MB,MB>& Ac, const Matrix<NB,MB>& Bc,
				Matrix<MB,MB>& Ad, Matrix<NB,MB>& Bd,
				const double Ts, const unsigned int Npade, const unsigned long Nint
			){
				Ad = expm(Ac*Ts,Npade);					// A行列の離散化
				Bd = integral_expm(Ac,Ts,Nint,Npade)*Bc;// B行列の離散化
			}
			
		private:
			Discret() = delete;		//!< コンストラクタ使用禁止
			~Discret() = delete;	//!< デストラクタ使用禁止
			Discret(const Discret&) = delete;	//!< コピーコンストラクタ使用禁止
			const Discret& operator=(const Discret&) = delete;	//!< 代入演算子使用禁止
	};
	
}

#endif

