//! @file DisturbanceObsrv.hh
//! @brief 外乱オブザーバクラス
//!
//! q軸電流とモータ側速度/位置からモータ側外乱トルクを推定します。
//!
//! @date 2019/02/27
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#ifndef DISTURBANCEOBSRV
#define DISTURBANCEOBSRV

#include <tuple>
#include "Matrix.hh"
#include "Discret.hh"
#include "ARCSeventlog.hh"

namespace ARCS {	// ARCS名前空間
	//! @brief 外乱オブザーバのタイプの定義
	enum class DObType {
		FULL_0TH,	//!< 同一次元0次外乱オブザーバ
		FULL_1ST	//!< 同一次元1次外乱オブザーバ
	};
	
	//! @brief 外乱オブザーバクラス
	//! @tparam	T	外乱オブザーバの形の設定
	template <DObType T>
	class DisturbanceObsrv {
		public:
			//! @brief コンストラクタ
			//! @param [in] TrqConst トルク定数 [Nm/A]
			//! @param [in] Inertia 慣性 [kgm^2]
			//! @param [in] Bandwidth 推定帯域 [rad/s]
			//! @param [in] SmplTime 制御周期 [s]
			DisturbanceObsrv(double TrqConst, double Inertia, double Bandwidth, double SmplTime)
				: Ktn(TrqConst), Jmn(Inertia), l1(-Bandwidth), l2(-Bandwidth), l3(-Bandwidth), Ts(SmplTime),
				  Ac(), Bc(), Ad(), Bd(), c(), u(), x(), x_next(), y()
			{
				// オブザーバの構成によって状態方程式を変える
				switch(T){
					// 同一次元0次オブザーバの場合
					case DObType::FULL_0TH:
						// 連続系A行列の設定
						Ac.Set(
							l1 + l2,  -1.0/Jmn,
							Jmn*l1*l2,       0
						);
						// 連続系B行列の設定
						Bc.Set(
							Ktn/Jmn,  - l1 - l2,
								  0, -Jmn*l1*l2
						);
						// C行列の設定
						c.Set(
							0, 1
						);
						break;
					
					// 同一次元1次オブザーバの場合
					case DObType::FULL_1ST:
						// 連続系のA行列
						Ac.Set(
							l1 + l2 + l3               , -1.0/Jmn,  0,
							Jmn*(l1*l2 + l2*l3 + l3*l1), 0       ,  1,
							-Jmn*l1*l2*l3              , 0       ,  0
						);
						// 連続系のB行列
						Bc.Set(
							Ktn/Jmn, -( l1 + l2 + l3 ),
							0      , -Jmn*( l1*l2 + l2*l3 + l3*l1 ),
							0      , Jmn*l1*l2*l3
						);
						// C行列
						c.Set(
							0,  1,  0
						);
						break;
					
					// それ以外
					default:
						arcs_assert(false);	// ここには来ない
						break;
				}
				
				// 離散化
				std::tie(Ad,Bd) = Discret::GetDiscSystem(Ac,Bc,Ts);
				PassedLog();
			}
			
			//! @brief ムーブコンストラクタ
			DisturbanceObsrv(DisturbanceObsrv&& right)
				: Ktn(right.Ktn), Jmn(right.Jmn), l1(right.l1), l2(right.l2), l3(right.l3), Ts(right.Ts),
				  Ac(right.Ac), Bc(right.Bc), Ad(right.Ad), Bd(right.Bd), c(right.c),
				  u(right.u), x(right.x), x_next(right.x_next), y(right.y)
			{
				
			}
			
			//! @brief デストラクタ
			~DisturbanceObsrv(){
				PassedLog();
			}

			//! @brief 外乱トルクを推定する関数
			//! @param [in] Current 電流 [A]
			//! @param [in] MotorSpeed モータ側速度 [rad/s]
			double GetDistTorque(double Current, double MotorSpeed){
				// 入力ベクトルの設定
				u.Set(
					Current,
					MotorSpeed
				);
				// 離散系状態方程式の計算
				x_next = Ad*x + Bd*u;
				y = c*x;
				// 状態ベクトルの更新
				x = x_next;
				// 出力ベクトルから抽出して返す
				return y.GetElement(1,1);
			}
			
			//! @brief 状態ベクトルをクリアする関数
			void ClearStateVector(void){
				x.FillAllZero();		// 状態ベクトルをクリア
				x_next.FillAllZero();	// 次の時刻の状態ベクトルもクリア
			}
			
		private:
			DisturbanceObsrv(const DisturbanceObsrv&) = delete;					//!< コピーコンストラクタ使用禁止
			const DisturbanceObsrv& operator=(const DisturbanceObsrv&) = delete;//!< 代入演算子使用禁止
			
			//!@ brief A行列の幅/高さを返す関数
			static constexpr unsigned int GetNA(void){
				switch(T){
					case DObType::FULL_0TH: return 2;
					case DObType::FULL_1ST: return 3;
				}
				return 0;
			}
			
			//!@ brief B行列の幅を返す関数
			static constexpr unsigned int GetNB(void){
				switch(T){
					case DObType::FULL_0TH: return 2;
					case DObType::FULL_1ST: return 2;
				}
				return 0;
			}
			
			//!@ brief B行列の高さを返す関数
			static constexpr unsigned int GetMB(void){
				switch(T){
					case DObType::FULL_0TH: return 2;
					case DObType::FULL_1ST: return 3;
				}
				return 0;
			}
			
			//!@ brief c行列の幅と状態ベクトルxの高さを返す関数
			static constexpr unsigned int GetCX(void){
				switch(T){
					case DObType::FULL_0TH: return 2;
					case DObType::FULL_1ST: return 3;
				}
				return 0;
			}
			
			double Ktn;	//!< [Nm/A]  トルク定数
			double Jmn;	//!< [kgm^2] モータ側慣性
			double l1;	//!< [rad/s] オブザーバの極1
			double l2;	//!< [rad/s] オブザーバの極2
			double l3;	//!< [rad/s] オブザーバの極3
			double Ts;	//!< [s] 制御周期
			Matrix<GetNA(),GetNA()> Ac;	//!<  連続系 A行列
			Matrix<GetNB(),GetMB()> Bc;	//!<  連続系 B行列
			Matrix<GetNA(),GetNA()> Ad;	//!<  離散系 A行列
			Matrix<GetNB(),GetMB()> Bd;	//!<  離散系 B行列
			Matrix<GetCX(),1> c;		//!<  C行列
			Matrix<1,2> u;				//!<  入力ベクトル
			Matrix<1,GetCX()> x;		//!<  状態ベクトル
			Matrix<1,GetCX()> x_next;	//!<  次の時刻の状態ベクトル
			Matrix<1,1> y;				//!<  出力ベクトル
	};
}

#endif

