//! @file Integrator.hh
//! @brief 積分器
//! @date 2017/05/15
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2017 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#include "Integrator.hh"

using namespace ARCS;

//! @brief コンストラクタ
//! @param[in]	SmplTime	制御周期 [s]
Integrator::Integrator(double SmplTime)
	: IntegType(BACKWARD_EULER),// 積分方法は後退オイラーをデフォルトで選択
	  Ts(SmplTime),				// [s] 制御周期の格納
	  uZ1(0), yZ1(0),
	  Ena(true)					// 積分器作動
{
	
}

//! @brief コンストラクタ(積分方法指定版)
//! @param[in]	SmplTime	制御周期 [s]
//! @param[in]	Type		積分方法
Integrator::Integrator(double SmplTime, enum IntType Type)
	: IntegType(Type),
	  Ts(SmplTime),		// [s] 制御周期の格納
	  uZ1(0), yZ1(0),
	  Ena(true)			// 積分器作動
{
	
}

//! @brief デストラクタ
Integrator::~Integrator(){
}

//! @brief 出力信号の取得
//! @param[in]	u	入力信号
//! @return 積分器出力
double Integrator::GetSignal(double u){
	double y = 0;
	
	if(Ena==false)return yZ1;	// 積分器停止指令の場合は積分動作を停止して積分値をそのまま出力
	
	// 積分方法によって計算を変える
	switch(IntegType){
		case FOWARD_EULER:
			y = Ts*uZ1 + yZ1;			// 前進オイラー(前進差分)
			break;
		case BACKWARD_EULER:
			y = Ts*u + yZ1;				// 後退オイラー(後退差分)
			break;
		case TRAPEZOIDAL:
			y = Ts/2.0*(u+uZ1) + yZ1;	// 台形積分(双1次変換, Tustin変換)
			break;
		default:
			y = Ts*u + yZ1;				// デフォルトは後退オイラー
			break;
	}
	
	// 状態変数の更新
	uZ1 = u;
	yZ1 = y;
	
	return y;
}

//! @brief 制御周期の再設定
//! @param[in] SmplTime 制御周期[s]
void Integrator::SetSmplTime(double SmplTime){
	Ts=SmplTime;	// [s] 制御周期の再設定
}

//! @brief 積分初期値の設定
//! @param[in] InitValue 初期値
void Integrator::SetInitial(double InitValue){
	yZ1=InitValue;
}

//! @brief 積分器を動かすか止めるかの選択
//! @param[in] Flag trueのとき積分器作動，falseのとき積分停止
void Integrator::Enable(bool Flag){
	Ena=Flag;
}

//! @brief すべての状態変数のリセット
void Integrator::ClearStateVars(void){
	uZ1=0;	// 状態変数1のゼロクリア
	yZ1=0;	// 状態変数2のゼロクリア
}

