//! @file MotorSimulator.cc
//! @brief モータシミュレータ
//!
//! モータを模擬する
//!
//! @date 2019/03/01
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#include <tuple>
#include "MotorSimulator.hh"
#include "Discret.hh"
#include "ARCSeventlog.hh"

using namespace ARCS;

//! @brief コンストラクタ
//! @param[in]	TrqConst	[Nm/A] トルク定数
//! @param[in]	MotorInert	[kgm^2]モータ慣性
//! @param[in]	MotorFric	[Nm/(rad/s)]モータ粘性
//! @param[in]	SmplTime	[s] サンプリング時間
MotorSimulator::MotorSimulator(double TrqConst, double MotorInert, double MotorFric, double SmplTime)
	: Kt(TrqConst), Jm(MotorInert), Dm(MotorFric), Ts(SmplTime),
	  Ac(), Bc(), Ad(), Bd(), u(), x(), x_next()
{
	// 連続系A行列の設定
	Ac.Set(
		-Dm/Jm,  0,
		     1,  0
	);
	
	// 連続系B行列の設定
	Bc.Set(
		Kt/Jm, -1.0/Jm,
		    0,       0
	);
	
	// 離散化
	std::tie(Ad,Bd) = Discret::GetDiscSystem(Ac,Bc,Ts);
	
	PassedLog();
}

//! @brief ムーブコンストラクタ
MotorSimulator::MotorSimulator(MotorSimulator&& r)
	: Kt(r.Kt), Jm(r.Jm), Dm(r.Dm), Ts(r.Ts),
	  Ac(r.Ac), Bc(r.Bc), Ad(r.Ad), Bd(r.Bd), u(r.u), x(r.x), x_next(r.x_next)
{
	
}

//! @brief デストラクタ
MotorSimulator::~MotorSimulator(){
	PassedLog();
}

//! @brief モータ速度と位置を計算する関数
//! @param[in]	current	[A] 電流
//! @param[in]	loadtorque	[Nm] 負荷トルク
//! @return	モータ速度, 位置
std::tuple<double,double> MotorSimulator::GetResponses(double current, double loadtorque){
	double velocity, position;
	
	// 入力ベクトルの設定
	u.Set(
		current,
		loadtorque
	);
	
	// 離散系状態方程式の計算
	x_next = Ad*x + Bd*u;
	
	// 状態ベクトルの更新
	x = x_next;
	
	// 状態ベクトルから抽出
	velocity = x.GetElement(1,1);
	position = x.GetElement(1,2);
	
	// 速度と位置を返す
	return std::forward_as_tuple(velocity, position);
}

//! @brief モータ慣性を設定する関数
void MotorSimulator::SetMotorInertia(double inertia){
	Jm = inertia;
}

//! @brief 状態ベクトルをリセットする関数
void MotorSimulator::ClearStateVector(void){
	x = Matrix<1,2>::zeros();
	x_next = Matrix<1,2>::zeros();
}

