//! @file MotorSimulator.hh
//! @brief モータシミュレータ
//!
//! モータを模擬する
//!
//! @date 2019/03/01
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#ifndef MOTORSIMULATOR
#define MOTORSIMULATOR

#include "Matrix.hh"

namespace ARCS {	// ARCS名前空間
	//! @brief モータシミュレータ
	class MotorSimulator {
		public:
			MotorSimulator(double TrqConst, double MotorInert, double MotorFric, double SmplTime);	//!< コンストラクタ
			MotorSimulator(MotorSimulator&& r);		//!< ムーブコンストラクタ
			~MotorSimulator();						//!< デストラクタ
			void SetMotorInertia(double inertia);	//!< 負荷側慣性を設定する関数
			std::tuple<double,double> GetResponses(double current, double loadtorque);	//!< モータ速度と位置を計算する関数
			void ClearStateVector(void);			//!< 状態ベクトルをリセットする関数
			
		private:
			MotorSimulator(const MotorSimulator&) = delete;					//!< コピーコンストラクタ使用禁止
			const MotorSimulator& operator=(const MotorSimulator&) = delete;//!< 代入演算子使用禁止
			
			double Kt;	//!< [Nm/A]	トルク定数
			double Jm;	//!< [kgm^2]	モータ側慣性モーメント
			double Dm;	//!< [Nm/(rad/s)]	モータ側粘性
			double Ts;	//!< [s]	サンプリング時間
			
			Matrix<2,2> Ac;	//!< 連続系 A行列
			Matrix<2,2> Bc;	//!< 連続系 B行列
			Matrix<2,2> Ad;	//!< 離散系 A行列
			Matrix<2,2> Bd;	//!< 離散系 B行列
			Matrix<1,2> u;	//!< 入力ベクトル
			Matrix<1,2> x;	//!< 状態ベクトル
			Matrix<1,2> x_next;	//!< 次の時刻の状態ベクトル
	};
}

#endif

