///! @file PNGraphics.cc
//! @brief PNG画像グラフィックス描画クラス
//! @date 2019/07/29
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2018 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the BSD License.
// For details, see the License.txt file.

#include <math.h>
#include <cassert>
#include "PNGraphics.hh"

using namespace ARCS;

//! @brief コンストラクタ
PNGraphics::PNGraphics(const std::string& FileName, int Width, int Height)
	: fp(nullptr), png_ptr(nullptr), info_ptr(nullptr), filename(FileName),
	  image(nullptr), width(Width), height(Height), size(0), length(0), IsFontDataLocked(false)
{
	fp = fopen(filename.c_str(), "wb"); // ファイルを開く
	png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);	// png_ptr構造体を確保・初期化
	info_ptr = png_create_info_struct(png_ptr); // info_ptr構造体を確保・初期化
	png_init_io(png_ptr, fp);			// libpngにfpを知らせ
	png_set_IHDR(						// IHDRチャンク情報を設定
		png_ptr,
		info_ptr,
		width,
		height,
		8,
		PNG_COLOR_TYPE_RGB_ALPHA,
		PNG_INTERLACE_NONE,
		PNG_COMPRESSION_TYPE_DEFAULT,
		PNG_FILTER_TYPE_DEFAULT
	);
	png_write_info(png_ptr, info_ptr);	// PNGファイルのヘッダを書き込みます
	png_set_invert_alpha(png_ptr);		// αチャネルをFFで透過にする(デフォは00で透過)
	png_set_bgr(png_ptr);				// uint32_tデータの順序をABGRからARGBに変更
	length = (long)width*(long)height;	// フレームバッファの長さを計算
	size = length*(long)32/8;			// 1画面データの大きさを計算 [bytes]
	image = (uint32_t*)malloc(length*sizeof(uint32_t));	// 画像バッファ用配列
	PrepareFontData(WHITE,BLACK);		// フォントデータの準備（初期値は白文字に黒背景）
}

//! @brief デストラクタ
PNGraphics::~PNGraphics(void){
	// 画像ファイルを書き出して閉じる
	for(long y=0;y<height;y++) png_write_row(png_ptr, (png_bytep)(&(image[width*y])));
	if(image != NULL) free(image);
	if (info_ptr != NULL) png_free_data(png_ptr, info_ptr, PNG_FREE_ALL, -1);
	if (png_ptr != NULL) png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
	if(fp != NULL) fclose(fp);
}

//! @brief 点(x,y)を描画する関数(バイナリデータ版)
//! @param[in]	x			x座標 [px]
//! @param[in]	y			y座標 [px]
//! @param[in]	ColorData	バイナリ色データ
inline void PNGraphics::DrawPoint(int x, int y, uint32_t ColorData){
	long i = (long)width*(long)y + (long)x;	// 座標からフレームバッファの要素番号を計算
	if(i<0)i=0;				// 要素番号リミッタ
	if(length<i)i=length-1;	// 要素番号リミッタ
	image[i] = ColorData;	// 画像データに書き込み
}

//! @brief 点(x,y)を描画する関数(色の名前版)
//! @param[in]	x			x座標 [px]
//! @param[in]	y			y座標 [px]
//! @param[in]	color		色の名前
void PNGraphics::DrawPoint(int x, int y, PGcolors color){
	DrawPoint(x, y, ColorNameToData(color));// バイナリ色データに変換して点描画関数を呼び出し
}

//! @brief 点(x,y)を描画する関数(RGB版)
//! @param[in]	x			x座標 [px]
//! @param[in]	y			y座標 [px]
//! @param[in]	r,g,b		RGB色データ
void PNGraphics::DrawPoint(int x, int y, double r, double g, double b){
	DrawPoint(x, y, RGBcolorToData(r,g,b));	// バイナリ色データに変換して点描画関数を呼び出し
}

//! @brief 直線(x1,y1)ー(x2,y2)を引く関数(バイナリデータ版)
//! @param[in]	x1, y1		始点座標 [px]
//! @param[in]	x2, y2		終点座標 [px]
//! @param[in]	ColorData	バイナリ色データ
void PNGraphics::DrawLine(int x1, int y1, int x2, int y2, uint32_t ColorData){
	// 垂直の線のとき(ブレゼンハムのアルゴリズムでは描けないので，別に実装)
	if(x1 == x2){
		int ysta, yend;
		if(y1 < y2){
			// y1→y2で増加する場合
			ysta = y1;
			yend = y2;
		}else{
			// y2→y1で増加する場合
			ysta = y2;
			yend = y1;
		}
		for(int y=ysta;y<=yend;y++)DrawPoint(x1, y, ColorData);	// 直線に沿って点を描画
		return;
	}
	
	// 水平の線のとき(ブレゼンハムのアルゴリズムでも描けるが，高速化のために別に実装)
	if( y1 == y2 ){
		int xsta, xend;
		if(x1 < x2){
			// x1→x2で増加する場合
			xsta = x1;
			xend = x2;
		}else{
			// x2→x1で増加する場合
			xsta = x2;
			xend = x1;
		}
		for(int x=xsta;x<=xend;x++)DrawPoint(x, y1, ColorData);	// 直線に沿って点を描画
		return;
	}
	
	// 以下はブレゼンハムのアルゴリズム
	bool IsHighGradient = std::abs(x2 - x1) < std::abs(y2 - y1);	// 直線の傾きが45度を境に急勾配かどうかのフラグ
	if(IsHighGradient == true){		// 45度を境に急勾配のとき
		std::swap(x1, y1);	// x1とy1を入れ替え
		std::swap(x2, y2);	// x2とy2を入れ替え
	}
	if(x2 < x1){	// 始点の方が終点よりも右のとき
		std::swap(x1, x2);
		std::swap(y1, y2);
	}
	int Dx = x2 - x1;
	int Dy = std::abs(y2 - y1);
	int Error = Dx/2;
	int ystep;
	if(y1 < y2){
		ystep = 1;
	}else{
		ystep = -1;
	}
	int y = y1;
	for(int x=x1;x<=x2;x++){
		if(IsHighGradient == true){
			DrawPoint(y, x, ColorData);
		}else{
			DrawPoint(x, y, ColorData);
		}
		Error -= Dy;
		if(Error < 0){
			y += ystep;
			Error += Dx;
		}
	}
}

//! @brief 直線(x1,y1)ー(x2,y2)を引く関数(色の名前版)
//! @param[in]	x1, y1		始点座標 [px]
//! @param[in]	x2, y2		終点座標 [px]
//! @param[in]	color		色の名前
void PNGraphics::DrawLine(int x1, int y1, int x2, int y2, PGcolors color){
	DrawLine(x1, y1, x2, y2, ColorNameToData(color));
}

//! @brief 直線(x1,y1)ー(x2,y2)を引く関数(RGB版)
//! @param[in]	x1, y1		始点座標 [px]
//! @param[in]	x2, y2		終点座標 [px]
//! @param[in]	r, g, b		RGB色データ
void PNGraphics::DrawLine(int x1, int y1, int x2, int y2, double r, double g, double b){
	DrawLine(x1, y1, x2, y2, RGBcolorToData(r,g,b));
}

//! @brief 階段直線(x1,y1)ー(x2,y2)を引く関数(バイナリデータ版)
//! @param[in]	x1, y1		始点座標 [px]
//! @param[in]	x2, y2		終点座標 [px]
//! @param[in]	ColorData	バイナリ色データ
void PNGraphics::DrawStairs(int x1, int y1, int x2, int y2, uint32_t ColorData){
	DrawLine(x1, y1, x2, y1, ColorData);
	DrawLine(x2, y1, x2, y2, ColorData);
}

//! @brief 階段直線(x1,y1)ー(x2,y2)を引く関数(色の名前版)
//! @param[in]	x1, y1		始点座標 [px]
//! @param[in]	x2, y2		終点座標 [px]
//! @param[in]	color		色の名前
void PNGraphics::DrawStairs(int x1, int y1, int x2, int y2, PGcolors color){
	DrawStairs(x1, y1, x2, y2, ColorNameToData(color));
}

//! @brief 階段直線(x1,y1)ー(x2,y2)を引く関数(RGB版)
//! @param[in]	x1, y1		始点座標 [px]
//! @param[in]	x2, y2		終点座標 [px]
//! @param[in]	r, g, b		RGB色データ
void PNGraphics::DrawStairs(int x1, int y1, int x2, int y2, double r, double g, double b){
	DrawStairs(x1, y1, x2, y2, RGBcolorToData(r,g,b));
}

//! @brief 長方形の描画をする関数(バイナリ色データ版)（色は塗らない）
//! @param[in] x, y			長方形左上の座標
//! @param[in] w, h			長方形の幅，長方形の高さ
//! @param[in] ColorData	バイナリ色データ
void PNGraphics::DrawRect(int x, int y, int w, int h, uint32_t ColorData){
	DrawLine(x  , y  , x+w, y  , ColorData);
	DrawLine(x+w, y  , x+w, y+h, ColorData);
	DrawLine(x+w, y+h, x  , y+h, ColorData);
	DrawLine(x  , y+h, x  , y  , ColorData);
}

//! @brief 長方形の描画をする関数(色の名前版)（色は塗らない）
//! @param[in] x, y			長方形左上の座標
//! @param[in] w, h			長方形の幅，長方形の高さ
//! @param[in] color		色の名前
void PNGraphics::DrawRect(int x, int y, int w, int h, PGcolors color){
	DrawRect(x, y, w, h, ColorNameToData(color));
}

//! @brief 長方形の描画をする関数(RGB版)（色は塗らない）
//! @param[in] x, y			長方形左上の座標
//! @param[in] w, h			長方形の幅，長方形の高さ
//! @param[in] r, g, b		RGB色データ
void PNGraphics::DrawRect(int x, int y, int w, int h, double r, double g, double b){
	DrawRect(x, y, w, h, RGBcolorToData(r,g,b));
}

//! @brief 長方形の範囲内に色を塗る関数(バイナリ色データ版) (色を塗る)
//! @param[in] x, y			長方形左上の座標
//! @param[in] w, h			長方形の幅，長方形の高さ
//! [param[in] ColorData	バイナリ色データ
void PNGraphics::DrawRectFill(int x, int y, int w, int h, uint32_t ColorData){
	long i = (long)width*(long)x + (long)y;	// 座標からフレームバッファの要素番号を計算
	for(int j=0;j<h;j++){
		// バイナリ色データを1ラインずつフレームバッファへ書き込む
		int sta = i + width*j;
		int end = sta + w;
		if(end <= length) std::fill(image + sta, image + end, ColorData);
	}
}

//! @brief 長方形の範囲内に色を塗る関数(色の名前版) (色を塗る)
//! @param[in] x, y			長方形左上の座標
//! @param[in] w			長方形の幅，長方形の高さ
//! @param[in] color		色の名前
void PNGraphics::DrawRectFill(int x, int y, int w, int h, PGcolors color){
	DrawRectFill(x, y, w, h, ColorNameToData(color));
}

//! @brief 長方形の範囲内に色を塗る関数(RGB色データ版) (色を塗る)
//! @param[in] x, y	長方形左上の座標
//! @param[in] w, h	長方形の幅，長方形の高さ
//! @param[in] r,g,b	色
void PNGraphics::DrawRectFill(int x, int y, int w, int h, double r, double g, double b){
	DrawRectFill(x, y, w, h, RGBcolorToData(r,g,b));
}

//! @brief 長方形の範囲内に色を塗る関数(RGBA色データ版) (色を塗る)
//! @param[in] x, y	長方形左上の座標
//! @param[in] w, h	長方形の幅，長方形の高さ
//! @param[in] r,g,b,a	色と透過率
void PNGraphics::DrawRectFill(int x, int y, int w, int h, double r, double g, double b, double a){
	DrawRectFill(x, y, w, h, RGBAcolorToData(r,g,b,a));
}

//! @brief 円の描画をする
//! @param[in]	cx, cy	円の中心座標
//! @param[in]	radius	半径，N：円の分割数
//! @param[in]	color	色の名前
void PNGraphics::DrawCircle(int cx, int cy, int radius, unsigned int N, PGcolors color){
	const double TwoPIdivN = 2.0*M_PI/(double)N;
	int x_prev = (double)radius + cx;
	int y_prev = cy;
	for(unsigned int i=0;i<=N;i++){
		double theta = TwoPIdivN*(double)i;
		int x = (double)radius*cos(theta) + cx;
		int y = (double)radius*sin(theta) + cy;
		DrawLine(x_prev,y_prev, x,y, color);
		x_prev = x;
		y_prev = y;
	}
}

//! @brief 円の描画をする
//! @param[in]	cx, cy	円の中心座標
//! @param[in]	radius	半径，N：円の分割数
//! @param[in]	r,g,b	色
void PNGraphics::DrawCircle(int cx, int cy, int radius, unsigned int N, double r, double g, double b){
	const double TwoPIdivN = 2.0*M_PI/(double)N;
	int x_prev = (double)radius + cx;
	int y_prev = cy;
	for(unsigned int i=0;i<=N;i++){
		double theta = TwoPIdivN*(double)i;
		int x = (double)radius*cos(theta) + cx;
		int y = (double)radius*sin(theta) + cy;
		DrawLine(x_prev,y_prev, x,y, r,g,b);
		x_prev = x;
		y_prev = y;
	}
}

//! @brief 色の名前からバイナリ色データに変換する関数
uint32_t PNGraphics::ColorNameToData(PGcolors color){
	return ColorSet32[color];
}

//! @brief 0～1の浮動小数点数値を赤緑青色の32bit輝度データに変える
uint32_t PNGraphics::RGBcolorToData(double Red, double Green, double Blue){
	uint32_t r = 0, g = 0, b = 0;
	r = DoubleToRedIntensity32(Red);	// 赤を変換
	g = DoubleToGreenIntensity32(Green);// 緑を変換
	b = DoubleToBlueIntensity32(Blue);	// 青を変換
	return r | g | b;					// ORをとって返す
}

//! @brief 0～1の浮動小数点数値を赤緑青透過色の32bit輝度データに変える
uint32_t PNGraphics::RGBAcolorToData(double Red, double Green, double Blue, double Alpha){
	uint32_t r = 0, g = 0, b = 0, a = 0;
	r = DoubleToRedIntensity32(Red);		// 赤を変換
	g = DoubleToGreenIntensity32(Green);	// 緑を変換
	b = DoubleToBlueIntensity32(Blue);		// 青を変換
	a = DoubleToAlphaIntensity32(Alpha);	// 青を変換
	return a | r | g | b;					// ORをとって返す
}

//! @brief 0～1の浮動小数点数値を白色の32bit輝度データに変える
uint32_t PNGraphics::DoubleToIntensity32(double u){
	if(1 < u)u = 1;
	if(u < 0)u = 0;
	return (uint32_t)((double)0x00FFFFFF*u);
}

//! @brief 0～1の浮動小数点数値を赤色の32bit輝度データに変える
uint32_t PNGraphics::DoubleToRedIntensity32(double u){
	if(1 < u)u = 1;
	if(u < 0)u = 0;
	return ((uint32_t)((double)0x000000FF*u)) << 16;
}

//! @brief 0～1の浮動小数点数値を緑色の32bit輝度データに変える
uint32_t PNGraphics::DoubleToGreenIntensity32(double u){
	if(1 < u)u = 1;
	if(u < 0)u = 0;
	return ((uint32_t)((double)0x000000FF*u)) << 8;
}

//! @brief 0～1の浮動小数点数値を青色の32bit輝度データに変える
uint32_t PNGraphics::DoubleToBlueIntensity32(double u){
	if(1 < u)u = 1;
	if(u < 0)u = 0;
	return ((uint32_t)((double)0x000000FF*u));
}

//! @brief 0～1の浮動小数点数値を透過値αの32bit輝度データに変える
uint32_t PNGraphics::DoubleToAlphaIntensity32(double u){
	if(1 < u)u = 1;
	if(u < 0)u = 0;
	return ((uint32_t)((double)0x000000FF*u)) << 24;
}

void PNGraphics::ClearScreen(void){
	// 全画面を消去
	memset(image,0,length-1);
}

void PNGraphics::ClearRect(int x, int y, int w, int h){
	// 長方形の範囲内を消去
	// x, y；長方形左上の座標，w；長方形の幅，h；長方形の高さ
	DrawRectFill(x,y,w,h,BLACK);
}

int PNGraphics::GetWidth(void) const {
	// 画面の幅の取得
	return width;
}

int PNGraphics::GetHeight(void) const {
	// 画面の高さの取得
	return height;
}

void PNGraphics::PrintValue(int x, int y, PGalign align, std::string format, double val){
	// 指定した書式で文字列を描画する x：[px] 横位置，y：[px] 縦位置，align：揃え位置，format：書式指定子，val：所望の数値
	// formatの書式指定子は printf関数 の場合の書き方と同等
	char str[TEXT_MAXLEN] = {'\0'};
	sprintf(str, format.c_str(), val);
	PrintText(x,y,align,str);
}

void PNGraphics::PrintText(int x, int y, PGalign align, std::string text){
	// 文字列を描画する x：[px]横位置，y：[px] 縦位置，align：揃え位置，text：所望の文字列
	
	// 文字列の揃え指定に従って座標にオフセットを持たせる
	switch(align){
		case ALIGN_LEFT:
			// 左揃えのときはなにもしない
			break;
		case ALIGN_CENTER:
			// 中央揃えのときは横位置から文字列の長さの半分を引く
			x = x - text.length()*(FONT_WIDTH + TEXT_INTERVAL)/2;
			break;
		case ALIGN_RIGHT:
			// 右揃えのときは横位置から文字列の長さを引く
			x = x - text.length()*(FONT_WIDTH + TEXT_INTERVAL);
			break;
	}
	
	// 1文字ずつ描画
	for(size_t i=0;i<text.length();i++){
		WriteFont(x + i*(FONT_WIDTH + TEXT_INTERVAL), y, (unsigned int)text.at(i));
	}
}

void PNGraphics::WriteFont(int x, int y, unsigned int ascii){
	// 1文字分のフォントデータをフレームバッファに書き込む
	// 注意！:フォントデータが準備中のときはなにもしない
	// x：[px] 横位置，y：[px] 縦位置，ascii：アスキーコード
	unsigned int j;
	unsigned int FontWidthInByte;		// フォントデータ1ラインのバイト数
	long i=(long)width*(long)y+(long)x;	// 座標からフレームバッファの要素番号を計算
	
	// フォントデータが準備中のときは何もせずに抜ける
	if(IsFontDataLocked == true)return;
	
	// アスキーコードが範囲外のときは範囲外用のフォントを使用
	if(ascii < FONT_FST_ASCII || FONT_END_ASCII < ascii)ascii = FONT_END_ASCII + 1;
	// アスキーコードからフォントデータの要素番号へ変換
	ascii = ascii - FONT_FST_ASCII;
	
	FontWidthInByte = sizeof(uint32_t)*FONT_WIDTH;	// フォントデータ1ラインのバイト数
	for(j=0;j<FONT_HEIGHT;j++){
		// フォントデータを1ラインずつフレームバッファへ書き込む
		memcpy(image + i + width*j, FontPrepared32[ascii][j], FontWidthInByte);
	}
}

void PNGraphics::PrepareFontData(PGcolors fore_color, PGcolors back_color){
	// 設定した色のフォントデータを準備
	// この関数は結構遅いはずなので注意。アホっぽいので設計から見直す必要あり。
	unsigned int i,j,k;
	
	IsFontDataLocked = true;	// フォントデータスピンロック
	
	// フォントデータの中身を書き換え
	for(i=0;i<FONT_NUM;i++){
		for(j=0;j<FONT_HEIGHT;j++){
			for(k=0;k<FONT_WIDTH;k++){
				if(FONT_DATA[i][j][k] == 1){
					FontPrepared32[i][j][k] = ColorSet32[fore_color];	// 前景色
				}else{
					FontPrepared32[i][j][k] = ColorSet32[back_color];	// 背景色
				}
			}
		}
	}
	
	IsFontDataLocked = false;	// フォントデータスピンロック解除
}

void PNGraphics::PrintAllFontData(){
	// 収録されているすべてのフォントデータを描画するテスト用関数
	for(unsigned int i=FONT_FST_ASCII;i<=FONT_END_ASCII;i++){
		WriteFont((i-FONT_FST_ASCII)*(FONT_WIDTH + TEXT_INTERVAL),0,i);
	}
}

// 色の定義 (32bit色深度用) 注意!!：enum PGcolors の定義と相違が無いようにせよ！
// ビットパターン MSB AAAAAAAA RRRRRRRR GGGGGGGG BBBBBBBB LSB (α：ビット31～25 赤：ビット24～16 緑：ビット15～8 青：ビット7～0)
// αは透過レベル
// 以下は基本色の定義  この他にも勝手に好きな色を作れる
const uint32_t PNGraphics::ColorSet32[NUM_COLOR_SET] = {
	0x00FF0000,	// RED
	0x0000FF00,	// GREEN
	0x000000FF,	// BLUE
	0x0000FFFF,	// CYAN
	0x00FF00FF,	// MAGENTA
	0x00FFFF00,	// YELLOW
	0x00FF8000,	// ORANGE
	0x00FFFFFF,	// WHITE
	0x00C0C0C0,	// GRAY75
	0x00808080,	// GRAY50
	0x00404040,	// GRAY25
	0x00000000,	// BLACK
	0xFF000000	// ALPHA
};

// フォントデータ
// (フォントが気に食わない人は各自で好きなように変えてネ)
const int PNGraphics::FONT_DATA[FONT_NUM][FONT_HEIGHT][FONT_WIDTH] = {
	{	// " " ASCII CODE = 32
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// "!" ASCII CODE = 33
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// """ ASCII CODE = 34
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// "#" ASCII CODE = 35
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{1, 1, 1, 1, 1},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{1, 1, 1, 1, 1},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0}
	},
	{	// "$" ASCII CODE = 36
		{0, 0, 1, 0, 0},
		{0, 1, 1, 1, 0},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 0},
		{0, 1, 1, 1, 0},
		{0, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{0, 1, 1, 1, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "%" ASCII CODE = 37
		{0, 1, 0, 0, 1},
		{1, 0, 1, 0, 1},
		{0, 1, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 1, 0},
		{0, 1, 1, 0, 1},
		{1, 0, 0, 1, 0}
	},
	{	// "&" ASCII CODE = 38
		{0, 0, 1, 0, 0},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 1, 0, 0},
		{1, 0, 1, 0, 1},
		{1, 0, 0, 1, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "'" ASCII CODE = 39
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// "(" ASCII CODE = 40
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 1, 0}
	},
	{	// ")" ASCII CODE = 41
		{0, 1, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0}
	},
	{	// "*" ASCII CODE = 42
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{1, 0, 1, 0, 1},
		{0, 1, 1, 1, 0},
		{1, 1, 1, 1, 1},
		{0, 1, 1, 1, 0},
		{1, 0, 1, 0, 1},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// "+" ASCII CODE = 43
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{1, 1, 1, 1, 1},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// "," ASCII CODE = 44
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0}
	},
	{	// "-" ASCII CODE = 45
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 1, 1, 1, 1},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// "." ASCII CODE = 46
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "/" ASCII CODE = 47
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{1, 0, 0, 0, 0}
	},
	{	// "0" ASCII CODE = 48
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 1, 1},
		{1, 0, 1, 0, 1},
		{1, 1, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "1" ASCII CODE = 49
		{0, 0, 1, 0, 0},
		{0, 1, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 1, 1, 0}
	},
	{	// "2" ASCII CODE = 50
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0},
		{1, 0, 0, 0, 1},
		{1, 1, 1, 1, 1}
	},
	{	// "3" ASCII CODE = 51
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 1, 1, 0},
		{0, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "4" ASCII CODE = 52
		{0, 0, 0, 1, 0},
		{0, 0, 1, 1, 0},
		{0, 0, 1, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{1, 0, 0, 1, 0},
		{1, 1, 1, 1, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0}
	},
	{	// "5" ASCII CODE = 53
		{1, 1, 1, 1, 1},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 1, 1, 1, 0},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "6" ASCII CODE = 54
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 0},
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "7" ASCII CODE = 55
		{1, 1, 1, 1, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "8" ASCII CODE = 56
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "9" ASCII CODE = 57
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// ":" ASCII CODE = 58
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// ";" ASCII CODE = 59
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0}
	},
	{	// "<" ASCII CODE = 60
		{0, 0, 0, 0, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 0, 1}
	},
	{	// "=" ASCII CODE = 61
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 1, 1, 1, 1},
		{0, 0, 0, 0, 0},
		{1, 1, 1, 1, 1},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// ">" ASCII CODE = 62
		{1, 0, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0},
		{1, 0, 0, 0, 0}
	},
	{	// "?" ASCII CODE = 63
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "@" ASCII CODE = 64
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 1, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 0, 1, 1},
		{1, 0, 0, 0, 0},
		{0, 1, 1, 1, 0}
	},
	{	// "A" ASCII CODE = 65
		{0, 0, 1, 0, 0},
		{0, 1, 0, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 1, 1, 1, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "B" ASCII CODE = 66
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 1, 1, 1, 0}
	},
	{	// "C" ASCII CODE = 67
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "D" ASCII CODE = 68
		{1, 1, 1, 0, 0},
		{1, 0, 0, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 1, 0},
		{1, 1, 1, 0, 0}
	},
	{	// "E" ASCII CODE = 69
		{1, 1, 1, 1, 1},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 1, 1, 1, 1}
	},
	{	// "F" ASCII CODE = 70
		{1, 1, 1, 1, 1},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0}
	},
	{	// "G" ASCII CODE = 71
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 1, 1, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "H" ASCII CODE = 72
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 1, 1, 1, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "I" ASCII CODE = 73
		{0, 1, 1, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 1, 1, 0}
	},
	{	// "J" ASCII CODE = 74
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "K" ASCII CODE = 75
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 1, 0},
		{1, 0, 1, 0, 0},
		{1, 1, 0, 0, 0},
		{1, 0, 1, 0, 0},
		{1, 0, 0, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "L" ASCII CODE = 76
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 1, 1, 1, 1}
	},
	{	// "M" ASCII CODE = 77
		{1, 0, 0, 0, 1},
		{1, 1, 0, 1, 1},
		{1, 1, 0, 1, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "N" ASCII CODE = 78
		{1, 0, 0, 0, 1},
		{1, 1, 0, 0, 1},
		{1, 1, 0, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 0, 1, 1},
		{1, 0, 0, 1, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "0" ASCII CODE = 79
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "P" ASCII CODE = 80
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0}
	},
	{	// "Q" ASCII CODE = 81
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 1, 0, 1, 1},
		{0, 1, 1, 1, 1}
	},
	{	// "R" ASCII CODE = 82
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 1, 1, 1, 0},
		{1, 0, 0, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "S" ASCII CODE = 83
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "T" ASCII CODE = 84
		{1, 1, 1, 1, 1},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "U" ASCII CODE = 85
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "V" ASCII CODE = 86
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "W" ASCII CODE = 87
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 1, 0, 1, 1},
		{1, 1, 0, 1, 1},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0}
	},
	{	// "X" ASCII CODE = 88
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "Y" ASCII CODE = 89
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "Z" ASCII CODE = 90
		{1, 1, 1, 1, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 1, 1, 1, 1}
	},
	{	// "[" ASCII CODE = 91
		{0, 1, 1, 1, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 1, 1, 0}
	},
	{	// "\" ASCII CODE = 92
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 0, 1}
	},
	{	// "]" ASCII CODE = 93
		{0, 1, 1, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 1, 0},
		{0, 1, 1, 1, 0}
	},
	{	// "^" ASCII CODE = 94
		{0, 0, 1, 0, 0},
		{0, 1, 0, 1, 0},
		{1, 0, 0, 0, 1},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// "_" ASCII CODE = 95
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 1, 1, 1, 1}
	},
	{	// "`" ASCII CODE = 96
		{0, 1, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// "a" ASCII CODE = 97
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 1, 1, 1, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 1, 1},
		{0, 1, 1, 0, 1}
	},
	{	// "b" ASCII CODE = 98
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 1, 1, 0},
		{1, 1, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "c" ASCII CODE = 99
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "d" ASCII CODE = 100
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 1, 1, 0, 1},
		{1, 0, 0, 1, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 1}
	},
	{	// "e" ASCII CODE = 101
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 1, 1, 1, 1},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "f" ASCII CODE = 102
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{1, 1, 1, 1, 1},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0}
	},
	{	// "g" ASCII CODE = 103
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 1, 1, 1, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 1},
		{0, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "h" ASCII CODE = 104
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 1, 1, 0},
		{1, 1, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "i" ASCII CODE = 105
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "j" ASCII CODE = 106
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "k" ASCII CODE = 107
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 1, 0},
		{1, 0, 1, 0, 0},
		{1, 1, 0, 0, 0},
		{1, 1, 0, 0, 0},
		{1, 0, 1, 0, 0},
		{1, 0, 0, 1, 0},
		{1, 0, 0, 0, 1}
	},
	{	// "l" ASCII CODE = 108
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0}
	},
	{	// "m" ASCII CODE = 109
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 1, 0, 1, 0},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1}
	},
	{	// "n" ASCII CODE = 110
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "o" ASCII CODE = 111
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 1, 1, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 0}
	},
	{	// "p" ASCII CODE = 112
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 0, 1, 1, 0},
		{1, 1, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 1, 1, 1, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0}
	},
	{	// "q" ASCII CODE = 113
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 1, 1, 0, 1},
		{1, 0, 0, 1, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 1, 1, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1}
	},
	{	// "r" ASCII CODE = 114
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 0, 1, 1, 0},
		{1, 1, 0, 0, 1},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0}
	},
	{	// "s" ASCII CODE = 115
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 1, 1, 1, 1},
		{1, 0, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{0, 1, 1, 1, 0},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 0, 1},
		{1, 1, 1, 1, 0}
	},
	{	// "t" ASCII CODE = 116
		{0, 0, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{1, 1, 1, 1, 1},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 1, 1}
	},
	{	// "u" ASCII CODE = 117
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 1, 1},
		{0, 1, 1, 0, 1}
	},
	{	// "v" ASCII CODE = 118
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "w" ASCII CODE = 119
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{1, 0, 1, 0, 1},
		{0, 1, 0, 1, 0}
	},
	{	// "x" ASCII CODE = 120
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 1, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1}
	},
	{	// "y" ASCII CODE = 121
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 0, 0, 0, 1},
		{1, 0, 0, 0, 1},
		{0, 1, 0, 1, 0},
		{0, 1, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "z" ASCII CODE = 122
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{1, 1, 1, 1, 1},
		{0, 0, 0, 0, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0},
		{1, 0, 0, 0, 0},
		{1, 1, 1, 1, 1}
	},
	{	// "{" ASCII CODE = 123
		{0, 0, 1, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 0, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 1, 0}
	},
	{	// "|" ASCII CODE = 124
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0}
	},
	{	// "}" ASCII CODE = 125
		{0, 1, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 0, 1, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 0, 1, 0, 0},
		{0, 1, 1, 0, 0}
	},
	{	// "~" ASCII CODE = 126
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 1, 0, 0, 0},
		{1, 0, 1, 0, 1},
		{0, 0, 0, 1, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0}
	},
	{	// "■" ASCII CODE = 範囲外
		{1, 1, 1, 1, 1},
		{1, 1, 1, 1, 1},
		{1, 1, 1, 1, 1},
		{1, 1, 1, 1, 1},
		{1, 1, 1, 1, 1},
		{1, 1, 1, 1, 1},
		{1, 1, 1, 1, 1},
		{1, 1, 1, 1, 1},
		{1, 1, 1, 1, 1}
	}
};

