///! @file PNGraphics.cc
//! @brief PNG画像グラフィックス描画クラス
//! @date 2019/07/29
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the BSD License.
// For details, see the License.txt file.

#ifndef PNGRAPHICS
#define PNGRAPHICS

#include <string>
#include <png.h>

namespace ARCS {	// ARCS名前空間
	class PNGraphics {
		// フレームバッファクラス
		public:
			// 色の定義  注意!!：const int*_t ColorSet* の定義と相違が無いようにせよ！
			enum PGcolors {
				RED,	// 赤
				GREEN,	// 緑
				BLUE,	// 青
				CYAN,	// 水(シアン)
				MAGENTA,// 紫(マゼンタ)
				YELLOW,	// 黄(イエロー)
				ORANGE,	// 橙
				WHITE,	// 白
				GRAY75,	// 灰 輝度75％
				GRAY50,	// 灰 輝度50％
				GRAY25,	// 灰 輝度25％
				BLACK,	// 黒
				ALPHA	// 透明
			};
			// 文字列揃え位置の定義
			enum PGalign {
				ALIGN_LEFT,		// 文字列左揃え
				ALIGN_CENTER,	// 文字列中央揃え
				ALIGN_RIGHT		// 文字列右揃え
			};
			PNGraphics(const std::string& FileName, int Width, int Height);
			~PNGraphics(void);												// デストラクタ   PNG書き出し
			void DrawPoint(int x, int y, uint32_t ColorData);				//!< 点(x,y)を描画する関数(バイナリデータ版)
			void DrawPoint(int x, int y, PGcolors color);					//!< 点(x,y)を描画する関数(色の名前版)
			void DrawPoint(int x, int y, double r, double g, double b);		//!< 点(x,y)を描画する関数(RGB版)
			void DrawLine(int x1, int y1, int x2, int y2, uint32_t ColorData);						//!< 直線(x1,y1)ー(x2,y2)を引く関数(バイナリデータ版)
			void DrawLine(int x1, int y1, int x2, int y2, PGcolors color);							//!< 直線(x1,y1)ー(x2,y2)を引く関数(色の名前版)
			void DrawLine(int x1, int y1, int x2, int y2, double r, double g, double b);			//!< 直線(x1,y1)ー(x2,y2)を引く関数(RGB版)
			void DrawStairs(int x1, int y1, int x2, int y2, uint32_t ColorData);					//!< 階段直線(x1,y1)ー(x2,y2)を引く関数(バイナリデータ版)
			void DrawStairs(int x1, int y1, int x2, int y2, PGcolors color);						//!< 階段直線(x1,y1)ー(x2,y2)を引く関数(色の名前版)
			void DrawStairs(int x1, int y1, int x2, int y2, double r, double g, double b);			//!< 階段直線(x1,y1)ー(x2,y2)を引く関数(RGB版)
			void DrawRect(int x, int y, int w, int h, uint32_t ColorData);							//!< 長方形の描画をする関数(バイナリ色データ版)（色は塗らない）
			void DrawRect(int x, int y, int w, int h, PGcolors color);								//!< 長方形の描画をする関数(色の名前版)（色は塗らない）
			void DrawRect(int x, int y, int w, int h, double r, double g, double b);				//!< 長方形の描画をする関数(RGB色データ版)（色は塗らない）
			void DrawRectFill(int x, int y, int w, int h, uint32_t ColorData);						//!< 長方形の範囲内に色を塗る関数(バイナリ色データ版) (色を塗る)
			void DrawRectFill(int x, int y, int w, int h, PGcolors color);							//!< 長方形の範囲内に色を塗る関数(色の名前版) (色を塗る)
			void DrawRectFill(int x, int y, int w, int h, double r, double g, double b);			//!< 長方形の範囲内に色を塗る関数(RGB版) (色を塗る)
			void DrawRectFill(int x, int y, int w, int h, double r, double g, double b, double a);	//!< 長方形の範囲内に色を塗る関数(RGBA版) (色を塗る)
			void DrawCircle(int cx, int cy, int radius, unsigned int N, PGcolors color);			// 円の描画をする cx,cy：円の中心座標，radius：半径，N：円の分割数
			void DrawCircle(int cx, int cy, int radius, unsigned int N, double r, double g, double b);// 円の描画をする cx,cy：円の中心座標，radius：半径，N：円の分割数
			void ClearScreen(void);										// 画面の消去
			void ClearRect(int x, int y, int w, int h);					// 長方形の範囲内を消去
			int GetWidth(void) const;									// 画面の幅の取得
			int GetHeight(void) const;									// 画面の高さの取得
			void PrintValue(int x, int y, PGalign align, std::string format, double val);// 指定した書式で文字列を描画する x：[px] 横位置，y：[px] 縦位置，align：揃え位置，format：書式指定子，val：所望の数値 formatの書式指定子は printf関数 の場合の書き方と同等
			void PrintText(int x, int y, PGalign align, std::string text);// 文字列を描画する x：[px]横位置，y：[px] 縦位置，align：揃え位置，text：所望の文字列
			void WriteFont(int x, int y, unsigned int ascii);			// 1文字分のフォントデータをフレームバッファに書き込む x：[px] 横位置，y：[px] 縦位置，ascii：アスキーコード
			void PrepareFontData(PGcolors fore_color, PGcolors back_color);	// 設定した色のフォントデータを準備
			void PrintAllFontData();	// 収録されているすべてのフォントデータを描画するテスト用関数
		
		private:
			PNGraphics(const PNGraphics&);					// コピーコンストラクタ使用禁止
			const PNGraphics& operator=(const PNGraphics&);	// 代入演算子使用禁止
			
			// PNG関連の定義
			FILE *fp;				// PNGファイルポインタ
			png_structp png_ptr;	// PNG画像ポインタ
			png_infop info_ptr;		// PNG情報ポインタ
			
			// 画像情報関連の定義
			std::string filename;	// ファイル名
			uint32_t *image;		// 画像配列
			int width;				// [px]		横幅
			int height;				// [px]		高さ
			long size;				// [bytes]	画面の大きさ
			long length;			// 			画像配列の長さ
			
			// 色の定義
			static const unsigned int NUM_COLOR_SET = 15;	// 定義する色の数  注意!!：enum PGcolors の要素数と同一にせよ！
			static const uint32_t ColorSet32[NUM_COLOR_SET];// 色定義(32bit色深度用)
			
			// フォントの定義
			static const unsigned int FONT_NUM     = 96;// 収録しているフォントの数
			static const unsigned int FONT_WIDTH   = 5;	// [px] フォントの幅
			static const unsigned int FONT_HEIGHT  = 9;	// [px] フォントの高さ
			static const unsigned int FONT_FST_ASCII =  32;	// フォントデータに収録している最初の文字のアスキーコード
			static const unsigned int FONT_END_ASCII = 126;	// フォントデータに収録している最後の文字のアスキーコード
			static const int FONT_DATA[FONT_NUM][FONT_HEIGHT][FONT_WIDTH];// フォントデータ
			uint32_t FontPrepared32[FONT_NUM][FONT_HEIGHT][FONT_WIDTH];	// 準備済みのフォントデータ(32bit色深度用)
			bool IsFontDataLocked;	// フォントデータスピンロックフラグ
			
			// 文字列関連の定義
			static const unsigned int TEXT_INTERVAL = 1;	// [px] 文字の間隔
			static const unsigned int TEXT_MAXLEN = 256;	// 文字列の最大値
			
			// 基本的な関数
			
			// 色変換関連の関数
			static uint32_t ColorNameToData(PGcolors color);			//!> 色の名前からバイナリ色データに変換する関数
			static uint32_t RGBcolorToData(double Red, double Green, double Blue);					//!> 0～1の浮動小数点数値を赤緑青色の32bit輝度データに変える
			static uint32_t RGBAcolorToData(double Red, double Green, double Blue, double Alpha);	//!> 0～1の浮動小数点数値を赤緑青透過色の32bit輝度データに変える
			static uint32_t DoubleToIntensity32(double u);		// 0～1の浮動小数点数値を白色の32bit輝度データに変える
			static uint32_t DoubleToRedIntensity32(double u);	// 0～1の浮動小数点数値を赤色の32bit輝度データに変える
			static uint32_t DoubleToGreenIntensity32(double u);	// 0～1の浮動小数点数値を緑色の32bit輝度データに変える
			static uint32_t DoubleToBlueIntensity32(double u);	// 0～1の浮動小数点数値を青色の32bit輝度データに変える
			static uint32_t DoubleToAlphaIntensity32(double u);	// 0～1の浮動小数点数値を透過値の32bit輝度データに変える		
	};
}

#endif

