//! @file RingBuffer.cc
//! @brief リングバッファクラス(テンプレート版)
//! @date 2019/07/29
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the BSD License.
// For details, see the License.txt file.
//
// リングバッファの動作例
//                                       SetFirstValue (進む方向→)
//                                               |
// [0][j=i+1+k-N=1][2][3][4][5][6][7][8][9][10][i=11][j=i+1=12][13][j=i+1+k=14][15][N-1=16]  (N=17)
//          |                                             |              |
//          |                                     GetFinalValue          |
//    GetRelativeValue(k=6)                                   GetRelativeValue(k=2)
//

#ifndef RINGBUFFER
#define RINGBUFFER

#include <pthread.h>
#include <string>
#include "ARCSassert.hh"

namespace ARCS {	// ARCS名前空間
	//! @brief リングバッファクラス
	//! @tparam	T	型
	//! @tparam N	バッファサイズ
	template <typename T, unsigned long N>
	class RingBuffer {
		public:
			//! @brief コンストラクタ
			// Length；リングバッファの大きさ
			RingBuffer()
				: i(0), BufMutex()
			{
				pthread_mutex_init(&BufMutex, NULL);// Mutexの初期化
				ClearBuffer();						// バッファクリア
			}
			
			//! @brief デストラクタ
			~RingBuffer(){
				
			}
			
			//! @brief 値をバッファの先頭に格納する関数
			//! @param[in]	u	入力値
			void SetFirstValue(const T& u){
				pthread_mutex_lock(&BufMutex);	// Mutexロック
				++i;				// リングバッファカウンタ
				if(N <= i) i = 0;	// バッファの最後まで行ったらカウンタリセット
				Buffer[i] = u;		// バッファに入力値を格納
				pthread_mutex_unlock(&BufMutex);// Mutexロック解除
			}
			
			//! @brief バッファの最後尾から k だけ先頭側に進んだところの値を取り出す関数
			//! @param[in]	k	相対要素番号
			T GetRelativeValue(const unsigned long k){
				std::string ret;
				unsigned long j;
				pthread_mutex_lock(&BufMutex);	// Mutexロック
				j = i + 1 + k;			// 今のリングバッファカウンタに相対値を加算
				if(N <= j) j = j - N;	// バッファの最後を超えていたら，先頭＋はみ出た分にセット
				arcs_assert(j < N);
				ret = Buffer[j];
				pthread_mutex_unlock(&BufMutex);// Mutexロック解除
				return ret;
			}
			
			//! @brief バッファから最後尾の値を取り出す関数
			//! @return	出力値
			T GetFinalValue(void) const {
				std::string ret;
				unsigned long j;
				pthread_mutex_lock(&BufMutex);	// Mutexロック
				if(i == N - 1){	// バッファカウンタがリングの終端に達していたら，
					j = 0;		// リングの先端へ
				}else{
					j = i + 1;	// それ以外は最後尾へ
				}
				ret = Buffer[j];
				pthread_mutex_unlock(&BufMutex);// Mutexロック解除
				return ret;
			}
			
			//! @brief カウンタを任意値に設定する関数
			//! @param[in]	j	任意のカウント値
			void SetCounter(const unsigned long j){
				pthread_mutex_lock(&BufMutex);	// Mutexロック
				i = j;					// カウンタを任意値にセット
				if(N <= i) i = N - 1;	// バッファの最後を超えてたら終端でリミット
				pthread_mutex_unlock(&BufMutex);// Mutexロック解除
			}
			
			//! @brief カウンタをリセットする関数
			void ResetCounter(void){
				pthread_mutex_lock(&BufMutex);	// Mutexロック
				i = 0;
				pthread_mutex_unlock(&BufMutex);// Mutexロック解除
			}
			
			//! @brief バッファのゼロクリア
			void ClearBuffer(void){
				pthread_mutex_lock(&BufMutex);	// Mutexロック
				for(unsigned long j = 0; j < N; ++j) Buffer[j] = 0;
				pthread_mutex_unlock(&BufMutex);// Mutexロック解除
			}
			
		private:
			RingBuffer(const RingBuffer&) = delete;					// コピーコンストラクタ使用禁止
			const RingBuffer& operator=(const RingBuffer&) = delete;// 代入演算子使用禁止
			unsigned long i;			// リングバッファカウンタ
			T Buffer[N];				// リングバッファ
			pthread_mutex_t BufMutex;	// バッファアクセス用Mutex
	};
	
	//! @brief リングバッファクラス(std::string版部分特殊化)
	//! @tparam N	バッファサイズ
	template <unsigned long N>
	class RingBuffer<std::string, N> {
		public:
			//! @brief コンストラクタ
			// Length；リングバッファの大きさ
			RingBuffer()
				: i(0), BufMutex()
			{
				pthread_mutex_init(&BufMutex, NULL);// Mutexの初期化
				ClearBuffer();						// バッファクリア
			}
			
			//! @brief デストラクタ
			~RingBuffer(){
				
			}
			
			//! @brief 値をバッファの先頭に格納する関数
			//! @param[in]	u	入力値
			void SetFirstValue(const std::string& u){
				pthread_mutex_lock(&BufMutex);	// Mutexロック
				++i;				// リングバッファカウンタ
				if(N <= i) i = 0;	// バッファの最後まで行ったらカウンタリセット
				Buffer[i] = u;		// バッファに入力値を格納
				pthread_mutex_unlock(&BufMutex);// Mutexロック解除
			}
			
			//! @brief バッファの最後尾から k だけ先頭側に進んだところの値を取り出す関数
			//! @param[in]	k	相対要素番号
			std::string GetRelativeValue(const unsigned long k){
				std::string ret;
				unsigned long j;
				pthread_mutex_lock(&BufMutex);	// Mutexロック
				j = i + 1 + k;			// 今のリングバッファカウンタに相対値を加算
				if(N <= j) j = j - N;	// バッファの最後を超えていたら，先頭"0"＋はみ出た分にセット
				arcs_assert(j < N);
				ret = Buffer[j];
				pthread_mutex_unlock(&BufMutex);// Mutexロック解除
				return ret;
			}
			
			//! @brief バッファから最後尾の値を取り出す関数
			//! @return	出力値
			std::string GetFinalValue(void) const {
				std::string ret;
				unsigned long j;
				pthread_mutex_lock(&BufMutex);	// Mutexロック
				if(i == N - 1){	// バッファカウンタがリングの終端に達していたら，
					j = 0;		// リングの先端へ
				}else{
					j = i + 1;	// それ以外は最後尾へ
				}
				ret = Buffer[j];
				pthread_mutex_unlock(&BufMutex);// Mutexロック解除
				return ret;
			}
			
			//! @brief カウンタを任意値に設定する関数
			//! @param[in]	j	任意のカウント値
			void SetCounter(const unsigned long j){
				pthread_mutex_lock(&BufMutex);	// Mutexロック
				i = j;					// カウンタを任意値にセット
				if(N <= i) i = N - 1;	// バッファの最後を超えてたら終端でリミット
				pthread_mutex_unlock(&BufMutex);// Mutexロック解除
			}
			
			//! @brief カウンタをリセットする関数
			void ResetCounter(void){
				pthread_mutex_lock(&BufMutex);	// Mutexロック
				i = 0;
				pthread_mutex_unlock(&BufMutex);// Mutexロック解除
			}
			
			//! @brief バッファのゼロクリア
			void ClearBuffer(void){
				pthread_mutex_lock(&BufMutex);	// Mutexロック
				for(unsigned long j = 0; j < N; ++j) Buffer[j] = "";
				pthread_mutex_unlock(&BufMutex);// Mutexロック解除
			}
		
		private:
			RingBuffer(const RingBuffer&) = delete;					// コピーコンストラクタ使用禁止
			const RingBuffer& operator=(const RingBuffer&) = delete;// 代入演算子使用禁止
			unsigned long i;			// リングバッファカウンタ
			std::string Buffer[N];		// リングバッファ
			pthread_mutex_t BufMutex;	// バッファアクセス用Mutex
	};
}

#endif

