//! @file SFthread.cc
//! @brief 実時間スレッド生成・破棄クラス(SCHED_FIFO版, sleep完全不使用版, 関数オブジェクト版)
//!        実時間スレッドを生成・破棄する
//!        実際に計測された制御周期や計算消費時間も提供する
//! @date 2019/07/30
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#include <time.h>
#include <unistd.h>
#include <sys/mman.h>
#include <cmath>
#include "SFthread.hh"
#include "CPUSettings.hh"
#include "LinuxCommander.hh"

// ARCS組込み用マクロ
#ifdef IMPL_IN_ARCS
	// ARCSに組み込まれる場合
	#include "ARCSassert.hh"
	#include "ARCSeventlog.hh"
#else
	// ARCSに組み込まれない場合
	#define arcs_assert(a) (assert(a))
	#define PassedLog()
	#define EventLog(a)
	#define EventLogVar(a)
#endif

using namespace ARCS;

//! @brief コンストラクタ
//! @param[in] PeriodTime	制御周期
//! @param[in] FuncObject	制御用実行関数の関数オブジェクト
//! @param[in] CPUno		使用するCPUコアの番号
SFthread::SFthread(const unsigned long PeriodTime, const std::function<bool(double,double,double)>& FuncObject, const int CPUno)
	: SyncMutex(PTHREAD_MUTEX_INITIALIZER),	// 同期用Mutex
	  SyncCond(PTHREAD_COND_INITIALIZER),	// 同期用条件
	  StateFlag(SFID_STOP),		// 動作状態フラグを「停止状態」に設定
	  Ts(PeriodTime),			// [ns] 制御周期の格納
	  FuncObj(FuncObject),		// 制御用実行関数への関数オブジェクトを格納
	  Time(),					// 時刻の初期化
	  ActPeriodicTime(),		// 周期時間の初期化
	  ComputationTime(),		// 消費時間の初期化
	  ThreadID(0),				// スレッド識別子の初期化
	  ThreadParam(),			// スレッドパラメータ
	  MaxMemo(0),				// サンプリング時間最大値計算用
	  MinMemo(PeriodTime*1e-9),	// サンプリング時間最小値計算用
	  n(0),						// 3σ計算用
	  u(0),		// 3σ計算用
	  s2(0)						// 3σ計算用
{
	// 実時間スレッドの生成と優先度の設定
	PassedLog();
	pthread_mutex_init(&SyncMutex, nullptr);	// 同期用Mutexの初期化
	pthread_cond_init(&SyncCond, nullptr);		// 同期用条件の初期化
	pthread_create(&ThreadID, NULL, (void*(*)(void*))RealTimeThread, this);	// スレッド生成
	CPUSettings::SetCPUandPolicy(ThreadID, CPUno, SCHED_FIFO);				// CPUコアの割り当てとスケジューリングポリシーの設定
	SetKernelParameters();		// カーネルパラメータをリアルタイム用に設定
	PassedLog();
}

//! @brief コンストラクタ(制御用実行関数の関数オブジェクトを後で設定する場合)
//! @param[in] PeriodTime	制御周期
//! @param[in] CPUno		使用するCPUコアの番号
SFthread::SFthread(const unsigned long PeriodTime, const int CPUno)
	: SyncMutex(PTHREAD_MUTEX_INITIALIZER),	// 同期用Mutex
	  SyncCond(PTHREAD_COND_INITIALIZER),	// 同期用条件
	  StateFlag(SFID_STOP),		// 動作状態フラグを「停止状態」に設定
	  Ts(PeriodTime),			// [ns] 制御周期の格納
	  FuncObj(),				// 制御用実行関数への関数オブジェクトを格納
	  Time(),					// 時刻の初期化
	  ActPeriodicTime(),		// 周期時間の初期化
	  ComputationTime(),		// 消費時間の初期化
	  ThreadID(0),				// スレッド識別子の初期化
	  ThreadParam(),			// スレッドパラメータ
	  MaxMemo(0),				// サンプリング時間最大値計算用
	  MinMemo(PeriodTime*1e-9),	// サンプリング時間最小値計算用
	  n(0),						// 3σ計算用
	  u(0),		// 3σ計算用
	  s2(0)						// 3σ計算用
{
	// 実時間スレッドの生成と優先度の設定
	PassedLog();
	pthread_mutex_init(&SyncMutex, nullptr);	// 同期用Mutexの初期化
	pthread_cond_init(&SyncCond, nullptr);		// 同期用条件の初期化
	pthread_create(&ThreadID, NULL, (void*(*)(void*))RealTimeThread, this);	// スレッド生成
	CPUSettings::SetCPUandPolicy(ThreadID, CPUno, SCHED_FIFO);				// CPUコアの割り当てとスケジューリングポリシーの設定
	SetKernelParameters();		// カーネルパラメータをリアルタイム用に設定
	PassedLog();
}

//! @brief ムーブコンストラクタ
//! @param[in]	r 右辺値
SFthread::SFthread(SFthread&& r)
	: SyncMutex(r.SyncMutex),				// 同期用Mutex
	  SyncCond(r.SyncCond),					// 同期用条件
	  StateFlag(r.StateFlag),				// 動作状態フラグ
	  Ts(r.Ts),								// [ns] 制御周期
	  FuncObj(r.FuncObj),					// 制御用実行関数への関数オブジェクト
	  Time(r.Time),							// 時刻
	  ActPeriodicTime(r.ActPeriodicTime),	// 周期時間
	  ComputationTime(r.ComputationTime),	// 消費時間
	  ThreadID(r.ThreadID),					// スレッド識別子
	  ThreadParam(r.ThreadParam),			// スレッドパラメータ
	  MaxMemo(r.MaxMemo),					// サンプリング時間最大値計算用
	  MinMemo(r.MinMemo),					// サンプリング時間最小値計算用
	  n(r.n),								// 3σ計算用
	  u(r.u),								// 3σ計算用
	  s2(r.s2)								// 3σ計算用
{
	
}

//! @brief デストラクタ
SFthread::~SFthread(){
	PassedLog();
	pthread_cancel(ThreadID);	// 実時間スレッド終了＆破棄
	ResetKernelParameters();	// カーネルパラメータを元に戻す
	PassedLog();
}

//! @brief 実時間スレッドから呼び出す関数を設定する関数
//! @param[in]	FuncObject	関数オブジェクト
void SFthread::SetRealtimeFunction(std::function<bool(double,double,double)> FuncObject){
	FuncObj = FuncObject;	// 関数オブジェクトをセット
}

//! @brief 実時間スレッド
//! @param[in]	p	クラスメンバアクセス用ポインタ
void SFthread::RealTimeThread(SFthread *p){
	// デストラクタが呼ばれるまで繰り返し続ける
	while(1){
		// 動作状態が「開始」に設定されるまで待機
		pthread_mutex_lock(&(p->SyncMutex));	// Mutexロック
		while(p->StateFlag != SFID_START){
			pthread_cond_wait(&(p->SyncCond), &(p->SyncMutex));	// 状態が更新されるまで待機
		}
		pthread_mutex_unlock(&(p->SyncMutex));	// Mutexアンロック
		
		mlockall(MCL_FUTURE);					// メモリロック
		pthread_mutex_lock(&(p->SyncMutex));	// Mutexロック
		p->StateFlag = SFID_RUN;				// 動作状態フラグを「動作中」に設定
		pthread_cond_signal(&(p->SyncCond));	// 実際の状態が更新されたことを上位系に知らせる
		pthread_mutex_unlock(&(p->SyncMutex));	// Mutexアンロック
		
		p->RealTimeLoop();	// 実時間ループの実行
		
		munlockall();							// メモリロック解除
		pthread_mutex_lock(&(p->SyncMutex));	// Mutexロック
		p->StateFlag = SFID_EXCMPL;				// 動作状態フラグを「終了動作完了」に設定
		pthread_cond_signal(&(p->SyncCond));	// 実際の状態が更新されたことを上位系に知らせる
		pthread_mutex_unlock(&(p->SyncMutex));	// Mutexアンロック
	}
}

//! 実時間ループ
void SFthread::RealTimeLoop(void){
	timespec InitTime = {0};		// ループに入る初期時刻
	timespec NextTime = {0};		// 次のループ開始時刻
	timespec TimeInWait = {0};		// 待機ループ内で取得する現在時刻
	timespec PeriodTime = nsec_to_timespec(Ts);	// 所望の制御周期
	timespec StartTime = {0};		// 開始時刻格納用
	timespec StartTimePrev = {0};	// 前回の開始時間格納用
	timespec EndTime = {0};			// 終了時刻格納用
	bool ClockOverride = false;		// 時刻待機のクロックオーバーライドフラグ
	
	clock_gettime(CLOCK_MONOTONIC, &InitTime);			// 初期開始時刻の取得
	StartTimePrev = timespec_sub(InitTime, PeriodTime);	// 実際の制御周期計算用の初期値設定
	
	// 実時間ループ
	while(StateFlag != SFID_STOP){	// 動作状態フラグが「停止」に設定されるまでループ
		// ここからリアルタイム空間
		clock_gettime(CLOCK_MONOTONIC, &StartTime);							// 開始時刻の取得
		Time = timespec_sub(StartTime, InitTime);							// 実際の時刻を計算
		ActPeriodicTime = timespec_sub(StartTime, StartTimePrev);			// 実際の周期時間を計算(timespec構造体は単純に減算できないことに注意)
		ClockOverride = !FuncObj(GetTime(), GetSmplTime(), GetCompTime());	// 制御用関数の実行(関数オブジェクトにより、ここで実際の制御関数が呼ばれる)
		StartTimePrev = StartTime;											// 次回用に今回の開始時刻を格納
		NextTime = timespec_add(StartTime, PeriodTime);						// 開始時刻に制御周期を加算して次の時刻を計算
		clock_gettime(CLOCK_MONOTONIC, &EndTime);							// 終了時刻の取得
		ComputationTime = timespec_sub(EndTime, StartTime);					// 消費時間を計算(timespec構造体は単純に減算できないことに注意)
		
		// 次の時刻になるまで待機
		while(StateFlag != SFID_STOP){
			clock_gettime(CLOCK_MONOTONIC, &TimeInWait);				// 現在時刻の取得
			if(timespec_lessthaneq(NextTime, TimeInWait) == true || ClockOverride == true){
				// 現在時刻と予め計算した次の時刻とを比較して，超えたら待機終了
				// もしくは，クロックオーバーライドフラグが立っていたら待機終了
				break;
			}
		}
		// リアルタイム空間ここまで
	}	
}

//! @brief スレッド実行を開始する関数
void SFthread::Start(void){
	pthread_mutex_lock(&SyncMutex);		// Mutexロック
	StateFlag = SFID_START;				// 開始が指令されたことを知らせる
	pthread_cond_signal(&SyncCond);		// 実際の状態が更新されたことをリアルタイムスレッドに知らせる
	pthread_mutex_unlock(&SyncMutex);	// Mutexアンロック
}

//! @brief スレッド実行が開始されるまで待機する関数
void SFthread::WaitStart(void){
	// 状態が「動作中」に設定されるまで待機
	pthread_mutex_lock(&SyncMutex);		// Mutexロック
	while(StateFlag != SFID_RUN){
		pthread_cond_wait(&SyncCond, &SyncMutex);	// 状態が更新されるまで待機
	}
	pthread_mutex_unlock(&SyncMutex);	// Mutexアンロック
}

//! @biref スレッド実行を停止する関数
void SFthread::Stop(void){
	pthread_mutex_lock(&SyncMutex);		// Mutexロック
	StateFlag = SFID_STOP;				// 停止が指令されたことを知らせる
	pthread_cond_signal(&SyncCond);		// 実際の状態が更新されたことをリアルタイムスレッドに知らせる
	pthread_mutex_unlock(&SyncMutex);	// Mutexアンロック
}

//! @brief スレッド実行が停止されるまで待機する関数
void SFthread::WaitStop(void){
	// 状態が「終了動作完了」に設定されるまで待機
	pthread_mutex_lock(&SyncMutex);		// Mutexロック
	while(StateFlag != SFID_EXCMPL){
		pthread_cond_wait(&SyncCond, &SyncMutex);	// 状態が更新されるまで待機
	}
	pthread_mutex_unlock(&SyncMutex);	// Mutexアンロック
}

//! @brief スレッドをリセットする関数
void SFthread::Reset(void){
	timespec_clear(Time);			// 時刻をクリア
	timespec_clear(ActPeriodicTime);// 実際の周期時間をクリア
	timespec_clear(ComputationTime);// 消費時間をクリア
	MaxMemo = 0;		// 計測周期最大値をクリア
	MinMemo = Ts*1e-9;	// 計測周期最小値をクリア
	n = 0;				// 3σ計算用データ数カウンタをクリア
	u = 0;		// 3σ計算用逐次平均をクリア
	s2 = 0;				// 3σ計算用逐次分散をクリア
}

//! @brief 時刻を取得する関数
//! @return 時刻 [s]
double SFthread::GetTime(void) const {
	return timespec_to_sec(Time);			// [s] 時刻に変換して返す
}

//! @brief 計測された実際のサンプリング時間を取得する関数
//! @return 計測周期 [s]
double SFthread::GetSmplTime(void) const {
	return timespec_to_sec(ActPeriodicTime);// [s] 時刻に変換して返す
}

//! @brief 計測された消費時間を取得する関数
//! @return 計測消費時間 [s]
double SFthread::GetCompTime(void) const {
	return timespec_to_sec(ComputationTime);// [s] 時刻に変換して返す
}

//! @brief 計測された実際のサンプリング時間の最大値を取得する関数
//! @return 計測最大サンプリング時間 [s]
double SFthread::GetMaxTime(void){
	double TsZ0 = GetSmplTime();	// [s] 今のサンプリング時間を取得
	if(MaxMemo < TsZ0){
		MaxMemo = TsZ0;
	}
	return MaxMemo;
}

//! @brief 計測された実際のサンプリング時間の最小値を取得する関数
//! @return 計測最小サンプリング時間 [s]
double SFthread::GetMinTime(void){
	double TsZ0 = GetSmplTime();	// [s] 今のサンプリング時間を取得
	if(TsZ0 < MinMemo && 1e-6 < TsZ0){
		MinMemo = TsZ0;
	}
	return MinMemo;
}

//! @brief 計測された実際のサンプリング時間の3σを取得する関数
//! @return 計測周期の3σ値 [s]
double SFthread::GetSmplTime3sigma(void){
	double x = GetSmplTime();					// [s] 今のサンプリング時間を取得
	double u_next = (n*u + x)/((double)n + 1.0);// 逐次平均の計算
	double s2_next = (n*(s2 + u*u) + x*x)/((double)n + 1.0) - u_next*u_next;	// 逐次分散の計算
	++n;				// データ数カウンタ
	u = u_next;			// 平均値の更新
	s2 = s2_next;		// 分散の更新
	return 3.0*sqrt(s2);// 3σを返す
}

//! @brief timespec構造体用の時刻を加算する関数 a + b
//! @param[in]	a	timespec時刻
//! @param[in]	b	timespec時刻
//! @return 加算結果
timespec SFthread::timespec_add(const timespec& a, const timespec& b){
	timespec ret;
	ret.tv_sec = a.tv_sec + b.tv_sec;		// aとbの秒カウンタを加算
	ret.tv_nsec = a.tv_nsec + b.tv_nsec;	// aとbのナノ秒カウンタを加算
	if(ONE_SEC_IN_NANO <= ret.tv_nsec){
		// ナノ秒カウンタが正方向に溢れる場合
		ret.tv_nsec -= ONE_SEC_IN_NANO;	// ナノ秒カウンタから1秒分引いて，
		ret.tv_sec++;					// 代わりに秒カウンタを加算
	}
	return ret;
}

//! @brief timespec構造体用の時刻を減算する関数 a - b
//! @param[in]	a	timespec時刻
//! @param[in]	b	timespec時刻
//! @return 減算結果
timespec SFthread::timespec_sub(const timespec& a, const timespec& b){
	timespec ret;
	ret.tv_sec = a.tv_sec - b.tv_sec;		// aとbの秒カウンタを減算
	ret.tv_nsec = a.tv_nsec - b.tv_nsec;	// aとbのナノ秒カウンタを減算
	if(ret.tv_nsec < 0){
		// ナノ秒カウンタが負方向に溢れる場合
		ret.tv_nsec += ONE_SEC_IN_NANO;	// ナノ秒カウンタに1秒分加えて，
		ret.tv_sec--;					// 代わりに秒カウンタを減算
	}
	return ret;
}

//! @brief timespec構造体用の時刻を比較する関数 a <= b
//! @param[in]	a	timespec時刻
//! @param[in]	b	timespec時刻
//! @return 比較結果
bool SFthread::timespec_lessthaneq(const timespec& a, const timespec& b){
	if(a.tv_sec != b.tv_sec){
		if(a.tv_sec < b.tv_sec){
			return true;
		}else{
			return false;
		}
	}else{
		if(a.tv_nsec <= b.tv_nsec){
			return true;
		}else{
			return false;
		}
	}
}

//! @brief timespec構造体用の時刻をクリアする関数 a = 0
//! @param[in]	a	timespec時刻
void SFthread::timespec_clear(timespec& a){
	a.tv_sec  = 0;	// 秒カウンタをクリア
	a.tv_nsec = 0;	// ナノ秒カウンタをクリア
}

//! @brief ナノ秒からtimespec構造体へ変換する関数
//! @param[in]	時刻 [ns]
//! @return		時刻(timespec構造体)
timespec SFthread::nsec_to_timespec(const unsigned long time){
	timespec ret;
	ret.tv_sec = time/ONE_SEC_IN_NANO;
	ret.tv_nsec = time - ret.tv_sec*ONE_SEC_IN_NANO;
	return ret;
}

//! @brief timespec構造体から秒へ変換する関数
//! @param[in]	時刻(timespec構造体)
//! @return		時刻 [s]
double SFthread::timespec_to_sec(const timespec& time){
	return (double)time.tv_sec + (double)time.tv_nsec*1e-9;	// [s] 時刻に変換して返す
}

//! @brief カーネルパラメータをリアルタイム用に設定する関数
void SFthread::SetKernelParameters(void){
	LinuxCommander::Execute("/bin/echo -1 > /proc/sys/kernel/sched_rt_runtime_us");			// CFSを無効
	LinuxCommander::Execute("/bin/echo 0 > /proc/sys/kernel/watchdog");						// 「BUG: soft lockup」警告防止
	LinuxCommander::Execute("/bin/echo 2147483647 > /proc/sys/kernel/sched_rt_period_us");	// リアルタイムタスク割り当て時間を最大化
	LinuxCommander::Execute("/bin/echo 0 > /proc/sys/kernel/timer_migration");				// タイマの移行を無効
	LinuxCommander::Execute("/bin/echo 0 > /proc/sys/kernel/sched_nr_migrate");				// プロセッサ間を移動できるタスク数をゼロにする
}

//! @brief カーネルパラメータを元に戻す関数
void SFthread::ResetKernelParameters(void){
	LinuxCommander::Execute("/bin/echo 1 > /proc/sys/kernel/watchdog");					// 「BUG: soft lockup」警告防止の解除
	LinuxCommander::Execute("/bin/echo 1000000 > /proc/sys/kernel/sched_rt_period_us");	// リアルタイムタスク割り当て時間を元に戻す
	LinuxCommander::Execute("/bin/echo 1 > /proc/sys/kernel/timer_migration");			// タイマの移行を有効
	LinuxCommander::Execute("/bin/echo 32 > /proc/sys/kernel/sched_nr_migrate");		// プロセッサ間を移動できるタスク数を既定値に戻す
}

