//! @file SFthread.hh
//! @brief 実時間スレッド生成・破棄クラス(SCHED_FIFO版)
//!        実時間スレッドを生成・破棄する
//!        実際に計測された制御周期や計算消費時間も提供する
//! @date 2019/07/30
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#ifndef SFTHREADING
#define SFTHREADING

#include <sys/time.h>
#include <pthread.h>
#include <functional>

namespace ARCS {	// ARCS名前空間
	//! @brief 実時間スレッド生成・破棄クラス
	class SFthread {
		public:
			//! @brief 動作状態の定義
			enum ThreadState {
				SFID_ERROR,	//!< エラー検出
				SFID_START,	//!< 開始
				SFID_RUN,	//!< 動作中
				SFID_STOP,	//!< 停止
				SFID_EXCMPL	//!< 終了動作完了
			};
			SFthread(const unsigned long PeriodTime, const std::function<bool(double,double,double)>& FuncObject, const int CPUno);//!< コンストラクタ
			SFthread(const unsigned long PeriodTime, const int CPUno);	//!< コンストラクタ(制御用実行関数の関数オブジェクトを後で設定する場合)
			SFthread(SFthread&& r);			//!< ムーブコンストラクタ
			~SFthread();					//!< デストラクタ
			void SetRealtimeFunction(std::function<bool(double,double,double)> FuncObject);	//!< 実時間スレッドから呼び出す関数を設定する関数
			void Start(void);				//!< スレッド実行を開始する関数
			void WaitStart(void);			//!< スレッド実行が開始されるまで待機する関数
			void Stop(void);				//!< スレッド実行を停止する関数
			void WaitStop(void);			//!< スレッド実行が停止されるまで待機する関数
			void Reset(void);				//!< スレッドをリセットする関数
			double GetTime(void) const;		//!< [s] 時刻を取得する関数
			double GetSmplTime(void) const;	//!< [s] 計測された実際のサンプリング時間を取得する関数
			double GetCompTime(void) const;	//!< [s] 計測された消費時間を取得する関数
			double GetMaxTime(void);		//!< [s] 計測された実際のサンプリング時間の最大値を取得する関数
			double GetMinTime(void);		//!< [s] 計測された実際のサンプリング時間の最小値を取得する関数
			double GetSmplTime3sigma(void);	//!< [s] 計測された実際のサンプリング時間の3σを取得する関数
			
		private:
			SFthread(const SFthread&) = delete;					//!< コピーコンストラクタ使用禁止
			const SFthread& operator=(const SFthread&) = delete;//!< 代入演算子使用禁止
			static const long ONE_SEC_IN_NANO = 1000000000;		//!< [ns] 1秒をナノ秒で表すと
			pthread_mutex_t SyncMutex;							//!< 同期用Mutex
			pthread_cond_t	SyncCond;							//!< 同期用条件
			volatile enum ThreadState StateFlag;				//!< 動作状態フラグ
			const unsigned long Ts;								//!< 制御周期
			std::function<bool(double,double,double)> FuncObj;	//!< 関数オブジェクト 引数(時刻, 計測周期, 消費時間)
			timespec Time;										//!< 計測された実際の時刻
			timespec ActPeriodicTime;							//!< 計測された実際の周期時間
			timespec ComputationTime;							//!< 計算によって消費された時間 (つまり ComputationTime < ActPeriodicTime でなければならない)
			static void RealTimeThread(SFthread *p);			//!< 実時間スレッド
			void RealTimeLoop(void);							//!< 実時間ループ
			pthread_t ThreadID;									//!< スレッド識別子
			struct sched_param ThreadParam;						//!< スレッドパラメータ
			double MaxMemo;										//!< [s] サンプリング時間最大値計算用
			double MinMemo;										//!< [s] サンプリング時間最小値計算用
			unsigned long n;//!< [-] 3σ計算用データ数カウンタ
			double u;		//!< [s] 3σ計算用データ数カウンタ
			double s2;		//!< [s] 3σ計算用逐次分散
			static timespec timespec_add(const timespec& a, const timespec& b);		//!< timespec構造体用の時刻を加算する関数 a + b
			static timespec timespec_sub(const timespec& a, const timespec& b);		//!< timespec構造体用の時刻を減算する関数 a - b
			static bool timespec_lessthaneq(const timespec& a, const timespec& b);	//!< timespec構造体用の時刻を比較する関数 a <= b
			static void timespec_clear(timespec& a);					//!< timespec構造体用の時刻をクリアする関数 a = 0
			static timespec nsec_to_timespec(const unsigned long time);	//!< ナノ秒からtimespec構造体へ変換する関数
			static double timespec_to_sec(const timespec& time);		//!< timespec構造体から秒へ変換する関数
			static void SetKernelParameters(void);		//!< カーネルパラメータをリアルタイム用に設定する関数
			static void ResetKernelParameters(void);	//!< カーネルパラメータを元に戻す関数
	};
}

#endif

