//! @file Statistics.cc
//! @brief 統計処理
//! 平均，分散，標準偏差，共分散，相関係数の計算
//!
//! @date 2019/02/22
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#include <math.h>
#include "Statistics.hh"

//! @brief 信号uの平均を計算する
double ARCS::Average(const double *u, unsigned long N){
	double ubar=0;
	unsigned long i;
	
	// 平均値を計算して返す
	for(i=0;i<N;i++)ubar=ubar+u[i];
	return ubar/(double)N;
}

//! @brief 信号uの分散を求める
double ARCS::Variance(const double *u, unsigned long N){
	double Ave, buff=0, y=0;
	unsigned long i;
	
	// uの平均を計算
	Ave=Average(u,N);
	
	// uから平均値を減算して標準偏差を計算して返す
	for(i=0;i<N;i++){
		buff=u[i]-Ave;
		y = y + buff*buff;
	}
	return y/(double)N;
}

//! @brief 信号uの標準偏差を求める
double ARCS::StandardDev(const double *u, unsigned long N){
	return sqrt(Variance(u,N));
}

//! @brief 信号u1と信号u2の間の共分散を計算する
double ARCS::Covariance(const double *u1, const double *u2, unsigned long N){
	double Ave1=0,Ave2=0,buff1=0,buff2=0,y=0;
	unsigned long i;
	
	// u1とu2の平均を計算
	Ave1=Average(u1,N);
	Ave2=Average(u2,N);
	
	// u1とu2から平均値を減算して共分散を計算して返す
	for(i=0;i<N;i++){
		buff1=u1[i]-Ave1;
		buff2=u2[i]-Ave2;
		y = y + buff1*buff2;
	}
	return y/(double)N;
}

//! @brief 信号u1と信号u2の間の相関係数を求める
double ARCS::Correlation(const double *u1, const double *u2, unsigned long N){
	double Ave1=0,Ave2=0,Cov=0,Std1=0,Std2=0,Corr=0;
	double buff1=0, buff2=0;
	unsigned long i=0;
	
	// u1とu2の平均を計算
	Ave1=Average(u1,N);
	Ave2=Average(u2,N);
	
	// u1とu2から平均値を減算して共分散と標準偏差を計算
	// (StandardDev関数とCovariance関数を使わないのは計算量を削減するため)
	for(i=0;i<N;i++){
		buff1=u1[i]-Ave1;
		buff2=u2[i]-Ave2;
		Cov  = Cov  + buff1*buff2;	// u1とu2共分散
		Std1 = Std1 + buff1*buff1;	// u1の標準偏差
		Std2 = Std2 + buff2*buff2;	// u2の標準偏差
	}
	
	// 最終的に相関係数を計算
	Corr=Cov/(sqrt(Std1)*sqrt(Std2));
	
	return Corr;
}

