//! @file TwoInertiaSimulator.cc
//! @brief 2慣性共振系シミュレータ
//!
//! 2慣性共振系を模擬する
//!
//! @date 2019/01/17
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#include <tuple>
#include "TwoInertiaSimulator.hh"
#include "Discret.hh"
#include "ARCSeventlog.hh"

using namespace ARCS;

//! @brief コンストラクタ
TwoInertiaSimulator::TwoInertiaSimulator(
	double TrqConst, double MotorInert, double LoadInert, double Spring,
	double MotorFric, double LoadFric, double GearRatio, double SmplTime
	)
	: Kt(TrqConst), Jm(MotorInert), Jl(LoadInert), Ks(Spring),
	  Dm(MotorFric), Dl(LoadFric), Rg(GearRatio), Ts(SmplTime),
	  Ac(), Bc(), Ad(), Bd(), C(), u(), x(), x_next(), y()
{
	// 連続系A行列の設定
	Ac.Set(
		-Dm/Jm,      0, -Ks/(Rg*Jm),
		     0, -Dl/Jl,       Ks/Jl,
		1.0/Rg,     -1,           0
	);
	
	// 連続系B行列の設定
	Bc.Set(
		Kt/Jm,       0,
		    0, -1.0/Jl,
		    0,       0
	);
	
	// C行列の設定
	C.Set(
		1,  0,  0,
		0,  1,  0
	);
	
	// 離散化
	std::tie(Ad,Bd) = Discret::GetDiscSystem(Ac,Bc,Ts);
	
	PassedLog();
}

//! @brief デストラクタ
TwoInertiaSimulator::~TwoInertiaSimulator(){
	PassedLog();
}

//! @brief モータ側＆負荷側速度を計算する関数
void TwoInertiaSimulator::GetVelocity(double current, double loadtorque, double& motorspeed, double& loadspeed){
	// 入力ベクトルの設定
	u.Set(
		current,
		loadtorque
	);
	
	// 離散系状態方程式の計算
	x_next = Ad*x + Bd*u;
	y = C*x;
	
	// 状態ベクトルの更新
	x = x_next;
	
	// 出力ベクトルから抽出
	motorspeed = y.GetElem(1,1);
	loadspeed = y.GetElem(2,1);
}

//! @brief モータ側＆負荷側速度を計算する関数(タプル版)
std::tuple<double,double> TwoInertiaSimulator::GetVelocity(double current, double loadtorque){
	double motorspeed, loadspeed;
	GetVelocity(current, loadtorque, motorspeed, loadspeed);
	return std::forward_as_tuple(motorspeed, loadspeed);
}

//! @brief 負荷側慣性を設定する関数
void TwoInertiaSimulator::SetLoadInertia(double inertia){
	Jl = inertia;
}

//! @brief ばね定数を設定する関数
void TwoInertiaSimulator::SetSpring(double stiffness){
	Ks = stiffness;
}

//! @brief 状態ベクトルをリセットする関数
void TwoInertiaSimulator::ResetStateVector(void){
	x = Matrix<1,3>::zeros();
	x_next = Matrix<1,3>::zeros();
}

