//! @file TwoInertiaSimulator.hh
//! @brief 2慣性共振系シミュレータ
//!
//! 2慣性共振系を模擬する
//!
//! @date 2018/12/30
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2018 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#ifndef TWOINERTIASIMULATOR
#define TWOINERTIASIMULATOR

#include "Matrix.hh"

namespace ARCS {	// ARCS名前空間
	//! @brief 2慣性共振系シミュレータ
	class TwoInertiaSimulator {
		public:
			TwoInertiaSimulator(
				double TrqConst, double MotorInert, double LoadInert, double Spring,
				double MotorFric, double LoadFric, double GearRatio, double SmplTime
			);	//!< コンストラクタ
			~TwoInertiaSimulator();	//!< デストラクタ
			void SetLoadInertia(double inertia);	//!< 負荷側慣性を設定する関数
			void SetSpring(double stiffness);		//!< ばね定数を設定する関数
			void GetVelocity(double current, double loadtorque, double& motorspeed, double& loadspeed);	//!< モータ側＆負荷側速度を計算する関数
			std::tuple<double,double> GetVelocity(double current, double loadtorque);	// モータ側＆負荷側速度を計算する関数(タプル版)
			void ResetStateVector(void);			//!< 状態ベクトルをリセットする関数
			
		private:
			TwoInertiaSimulator(const TwoInertiaSimulator&) = delete;					//!< コピーコンストラクタ使用禁止
			const TwoInertiaSimulator& operator=(const TwoInertiaSimulator&) = delete;	//!< 代入演算子使用禁止
			
			double Kt;	//!< [Nm/A]	トルク定数
			double Jm;	//!< [kgm^2]	モータ側慣性モーメント
			double Jl;	//!< [kgm^2]	負荷側慣性モーメント
			double Ks;	//!< [Nm/rad]	2慣性間の剛性
			double Dm;	//!< [Nm/(rad/s)]	モータ側粘性
			double Dl;	//!< [Nm/(rad/s)]	負荷側粘性
			double Rg;	//!< [-]	減速比
			double Ts;	//!< [s]	サンプリング時間
			
			Matrix<3,3> Ac;	//!< 連続系 A行列
			Matrix<2,3> Bc;	//!< 連続系 B行列
			Matrix<3,3> Ad;	//!< 離散系 A行列
			Matrix<2,3> Bd;	//!< 離散系 B行列
			Matrix<3,2> C;	//!< C行列
			Matrix<1,2> u;	//!< 入力ベクトル
			Matrix<1,3> x;	//!< 状態ベクトル
			Matrix<1,3> x_next;	//!< 次の時刻の状態ベクトル
			Matrix<1,2> y;	//!< 出力ベクトル
	};
}

#endif



