//! @file ControlFunctions.cc
//! @brief 制御用周期実行関数群クラス
//! @date 2019/07/30
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the BSD License.
// For details, see the License.txt file.

#include <cmath>
#include "ControlFunctions.hh"
#include "ScreenParams.hh"
#include "Interface.hh"
#include "GraphPlot.hh"
#include "ARCSprint.hh"
#include "ARCSassert.hh"
#include "Matrix.hh"
#include "SquareWave.hh"
#include "MotorSimulator.hh"
#include "DisturbanceObsrv.hh"

using namespace ARCS;

//! @brief スレッド間通信用グローバル変数の無名名前空間
namespace {
	std::array<double, ConstParams::ACTUATOR_MAX> PositionRes = {0};	//!< [rad] 位置応答
	std::array<double, ConstParams::ACTUATOR_MAX> CurrentRef  = {0};	//!< [A]   q軸電流指令
}

//! @brief 制御用周期実行関数1
//! @param[in]	t		時刻 [s]
//! @param[in]	Tact	計測周期 [s]
//! @param[in]	Tcmp	消費時間 [s]
//! @return		クロックオーバーライドフラグ (true = リアルタイムループ, false = 非リアルタイムループ)
bool ControlFunctions::ControlFunction1(double t, double Tact, double Tcmp){
	// 制御用定数設定
	const double Ts = ConstParams::SAMPLING_TIME[0]*1e-9;	// [s]			制御周期
	const double Kt[6] = {0.01,0.01,0.01,0.01,0.01,0.01};	// [Nm/A] 		トルク定数
	const double Jm[6] = {1e-4,1e-4,1e-4,1e-4,1e-4,1e-4};	// [kgm^2]		モータ慣性
	const double Dm[6] = {1e-4,1e-4,1e-4,1e-4,1e-4,1e-4};	// [Nm/(rad/s)]	モータ粘性
	const double gdis[6] = {300,300,300,300,300,300};		// [rad/s]		外乱オブザーバの推定帯域
	const double Kp[6] = {1,2,3,4,5,6};						// [-]			速度制御比例ゲイン
	
	// 制御用変数宣言
	static double iqref[6] = {0.1,0.1,0.1,0.1,0.1,0.1};	// [A]		q軸電流指令
	static double tload[6] = {0};	// [Nm]		負荷トルク
	static double tdis[6]  = {0};	// [Nm]		外乱トルク
	static double wm[6]    = {0};	// [rad/s]	モータ速度
	static double wmref[6] = {0};	// [rad/s]	モータ速度指令
	
	// 制御器等々の宣言
	// モータシミュレータの配列
	static MotorSimulator Motor[6] = {
		MotorSimulator(Kt[0], Jm[0], Dm[0], Ts),
		MotorSimulator(Kt[1], Jm[1], Dm[1], Ts),
		MotorSimulator(Kt[2], Jm[2], Dm[2], Ts),
		MotorSimulator(Kt[3], Jm[3], Dm[3], Ts),
		MotorSimulator(Kt[4], Jm[4], Dm[4], Ts),
		MotorSimulator(Kt[5], Jm[5], Dm[5], Ts)
	};
	// 外乱オブザーバの配列
	static DisturbanceObsrv<DObType::FULL_1ST> DOb[6] = {
		DisturbanceObsrv<DObType::FULL_1ST>(Kt[0], Jm[0], gdis[0], Ts),
		DisturbanceObsrv<DObType::FULL_1ST>(Kt[1], Jm[1], gdis[1], Ts),
		DisturbanceObsrv<DObType::FULL_1ST>(Kt[2], Jm[2], gdis[2], Ts),
		DisturbanceObsrv<DObType::FULL_1ST>(Kt[3], Jm[3], gdis[3], Ts),
		DisturbanceObsrv<DObType::FULL_1ST>(Kt[4], Jm[4], gdis[4], Ts),
		DisturbanceObsrv<DObType::FULL_1ST>(Kt[5], Jm[5], gdis[5], Ts)
	};
	
	if(CmdFlag == CTRL_INIT){
		// 初期化モード (ここは制御開始時/再開時に1度だけ呼び出される(非リアルタイム空間なので重い処理もOK))
		ScreenParams::InitOnlineSetVar();	// オンライン設定変数の初期値の設定
		Interface::ServoON();				// サーボON指令の送出
		for(size_t i = 0; i < 6; ++i){
			Motor[i].ClearStateVector();	// モータシミュレータの状態ベクトルのクリア (コーディングの例)
			DOb[i].ClearStateVector();		// 外乱オブザーバの状態ベクトルのクリア（コーディングの例）
		}
	}
	if(CmdFlag == CTRL_LOOP){
		// 周期モード (ここは制御周期 SAMPLING_TIME[0] 毎に呼び出される(リアルタイム空間なので処理は制御周期内に収めること))
		// リアルタイム制御ここから
		Interface::GetPosition(PositionRes);// [rad]位置応答の取得
		ScreenParams::GetOnlineSetVar();	// [-]  オンライン設定変数の読み込み
		
		// ここに所望の制御コードを書くがよい！
		// (下記は6軸単慣性系の速度制御のコーディングの例)
		for(size_t i = 0; i < 6; ++i){
			std::tie(wm[i], PositionRes[i]) = Motor[i].GetResponses(iqref[i], tload[i]);	// [rad/s,rad] モータシミュレータの計算
			tdis[i] = DOb[i].GetDistTorque(iqref[i], wm[i]);				// [Nm] 外乱オブザーバの計算
			wmref[i] = SquareWave(0.1, M_PI, t);							// [rad/s] 速度指令生成
			iqref[i] = (wmref[i] - wm[i])*Kp[i]*Jm[i]/Kt[i] + tdis[i]/Kt[i];// [A] P速度制御＋外乱オブザーバ加速度制御
			CurrentRef[i] = iqref[i];	// [A] 電流指令
		}
		
		Interface::SetCurrent(CurrentRef);	// [A] 電流指令の出力
		ScreenParams::SetVarIndicator(wmref[0], wm[0], wm[1], wm[2], wm[3], wm[4], wm[5], 0, 0, 0);// 任意変数インジケータ(変数0, ..., 変数9)
		GraphPlot::SetVars(0, wmref[0], wm[0], wm[1], wm[2], wm[3], wm[4], wm[5], 0);	// グラフプロット0 (グラフ番号, 変数0, ..., 変数7)
		GraphPlot::SetVars(1, 0, 0, 0, 0, 0, 0, 0, 0);	// グラフプロット1 (グラフ番号, 変数0, ..., 変数7)
		GraphPlot::SetVars(2, 0, 0, 0, 0, 0, 0, 0, 0);	// グラフプロット2 (グラフ番号, 変数0, ..., 変数7)
		GraphPlot::SetVars(3, 0, 0, 0, 0, 0, 0, 0, 0);	// グラフプロット3 (グラフ番号, 変数0, ..., 変数7)
		Initializing = false;							// 初期化中ランプ点灯用
		ExpData.SetData(t, wmref[0], wm[0], wm[1], wm[2], wm[3], wm[4], wm[5], 0, 0);	// CSV/DATデータ保存変数
		// リアルタイム制御ここまで
	}
	if(CmdFlag == CTRL_EXIT){
		// 終了処理モード (ここは制御終了時に1度だけ呼び出される(非リアルタイム空間なので重い処理もOK))
		Interface::SetZeroCurrent();// 電流指令を零に設定
		Interface::ServoOFF();		// サーボOFF信号の送出
	}
	return true;	// クロックオーバーライドフラグ(falseにすると次の周期時刻を待たずにスレッドが即刻動作する)
}

//! @brief 制御用周期実行関数2
//! @param[in]	t		時刻 [s]
//! @param[in]	Tact	計測周期 [s]
//! @param[in]	Tcmp	消費時間 [s]
//! @return		クロックオーバーライドフラグ (true = リアルタイムループ, false = 非リアルタイムループ)
bool ControlFunctions::ControlFunction2(double t, double Tact, double Tcmp){
	// 制御用定数宣言
	
	// 制御用変数宣言
	
	if(CmdFlag == CTRL_INIT){
		// 初期化モード (ここは制御開始時/再開時に1度だけ呼び出される(非リアルタイム空間なので重い処理もOK))
	}
	if(CmdFlag == CTRL_LOOP){
		// 周期モード (ここは制御周期 SAMPLING_TIME[1] 毎に呼び出される(リアルタイム空間なので処理は制御周期内に収めること))
	}
	if(CmdFlag == CTRL_EXIT){
		// 終了処理モード (ここは制御終了時に1度だけ呼び出される(非リアルタイム空間なので重い処理もOK))
	}
	return true;	// クロックオーバーライドフラグ(falseにすると次の周期時刻を待たずにスレッドが即刻動作する)
}

//! @brief 制御用周期実行関数3
//! @param[in]	t		時刻 [s]
//! @param[in]	Tact	計測周期 [s]
//! @param[in]	Tcmp	消費時間 [s]
//! @return		クロックオーバーライドフラグ (true = リアルタイムループ, false = 非リアルタイムループ)
bool ControlFunctions::ControlFunction3(double t, double Tact, double Tcmp){
	// 制御用定数宣言
	
	// 制御用変数宣言
	
	if(CmdFlag == CTRL_INIT){
		// 初期化モード (ここは制御開始時/再開時に1度だけ呼び出される(非リアルタイム空間なので重い処理もOK))
	}
	if(CmdFlag == CTRL_LOOP){
		// 周期モード (ここは制御周期 SAMPLING_TIME[2] 毎に呼び出される(リアルタイム空間なので処理は制御周期内に収めること))
	}
	if(CmdFlag == CTRL_EXIT){
		// 終了処理モード (ここは制御終了時に1度だけ呼び出される(非リアルタイム空間なので重い処理もOK))
	}
	return true;	// クロックオーバーライドフラグ(falseにすると次の周期時刻を待たずにスレッドが即刻動作する)
}

//! @brief 制御用変数値を更新する関数
void ControlFunctions::UpdateControlValue(void){
	// ARCS画面パラメータに値を書き込む
	ScreenParams::SetNetworkLink(NetworkLink);		// ネットワークリンクフラグを書き込む
	ScreenParams::SetInitializing(Initializing);	// ロボット初期化フラグを書き込む
	ScreenParams::SetCurrentAndPosition(CurrentRef, PositionRes);	// 電流指令と位置応答を書き込む
}

