//! @file Interface.cc
//! @brief インターフェースクラス
//! @date 2019/04/18
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the BSD License.
// For details, see the License.txt file.

#include <memory>
#include <unistd.h>
#include "Interface.hh"
#include "ARCSeventlog.hh"
#include "ARCSassert.hh"
#include "ARCSprint.hh"
#include "PCI-3340.hh"
#include "PCI-3343A.hh"
#include "PCI-3133.hh"
#include "PCI-3180.hh"
#include "PCI-6205C.hh"
#include "PCI-46610x.hh"
#include "WEF-6A.hh"

using namespace ARCS;

// インターフェースボードクラスの定義と実体
namespace {
	// ここにD/A，A/D，エンコーダボードクラス等々の定義と実体をスマートポインタで記述する
	std::unique_ptr<PCI3340>   DAC1;	// DACボード
	std::unique_ptr<PCI3343A>  DAC2;	// DACボード
	std::unique_ptr<PCI3133>   ADC1;	// ADCボード
	std::unique_ptr<PCI3180>   ADC2;	// ADCボード
	std::unique_ptr<PCI6205C>  ENC;		// ENCカウンタボード
	std::unique_ptr<PCI46610x> RS422CH1;// RS422シリアル通信ボード
	std::unique_ptr<WEF6A> ForceSensor;	// 6軸力覚センサ
}

//! @brief コンストラクタ
Interface::Interface(){
	PassedLog();
	
	// ここでD/A，A/D，エンコーダ，シリアル通信，力覚センサクラス等々を生成する
	DAC1 = std::make_unique<PCI3340>(0x00000000);		// DACボード
	DAC2 = std::make_unique<PCI3343A>(0x00000000);		// DACボード
	ADC1 = std::make_unique<PCI3133>(0x00000000);		// ADCボード
	ADC2 = std::make_unique<PCI3180>(0x00000000, PCI3180::RANGE_B_10V);			// ADCボード(バイポーラ±10V入力)
	ENC  = std::make_unique<PCI6205C>(0x0000, 0x0000, 0x0000, 0x0000, 8, true);	// ENCカウンタボード(使用CH8, 4逓倍有効)
	RS422CH1 = std::make_unique<PCI46610x>(0xfb9ff000);	// RS422シリアル通信ボード
	RS422CH1->SetConfig(
		PCI46610x::RATE_921_6kbps,	// ボーレート 921.6kbps
		PCI46610x::WIRE_4,			// 4線式
		PCI46610x::PARITY_DISABLE,	// パリティなし
		PCI46610x::STOPBIT_1,		// ストップビット1
		PCI46610x::WORDLENG_8		// ワード長8bit
	);
	ForceSensor = std::make_unique<WEF6A>(std::move(RS422CH1));
	
	PassedLog();
}

//! @brief デストラクタ
Interface::~Interface(){
	SetZeroCurrent();	// 念のためのゼロ電流指令
	PassedLog();
}

//! @brief サーボON信号を送出する関数
void Interface::ServoON(void){
	// ここにサーボアンプへのサーボON信号の送出シーケンスを記述する
	
}

//! @brief サーボOFF信号を送出する関数
void Interface::ServoOFF(void){
	// ここにサーボアンプへのサーボOFF信号の送出シーケンスを記述する
	
}

//! @brief 電流指令をゼロに設定する関数
void Interface::SetZeroCurrent(void){
	// ここにゼロ電流指令とサーボアンプの関係を列記する
	
}

//! @brief 位置応答を取得する関数
//! @param[out]	Position	位置応答
void Interface::GetPosition(std::array<double, ConstParams::ACTUATOR_MAX>& Position){
	// ここにエンコーダとPosition配列との関係を列記する
	
}

//! @brief トルク応答を取得する関数
//! @param[out]	Torque	トルク応答
void Interface::GetTorque(std::array<double, ConstParams::ACTUATOR_MAX>& Torque){
	// ここにトルクセンサとTorque配列との関係を列記する
	
}

//! @brief 電流応答を取得する関数
//! @param[out]	Current	電流応答
void Interface::GetCurrent(std::array<double, ConstParams::ACTUATOR_MAX>& Current){
	// ここに電流センサとCurrent配列との関係を列記する
	
}

//! @brief 電流指令を設定する関数
//! @param[in]	Current	電流指令
void Interface::SetCurrent(const std::array<double, ConstParams::ACTUATOR_MAX>& Current){
	// ここにCurrent配列とサーボアンプの関係を列記する
	
}

//! @brief 6軸力覚センサ応答を取得する関数
//! @param[out]	Fx-Fy 各軸の並進力 [N]
//! @param[out]	Mx-My 各軸のトルク [Nm]
void Interface::Get6axisForce(double& Fx, double& Fy, double& Fz, double& Mx, double& My, double& Mz){
	// ここに6軸力覚センサとForce配列との関係を列記する
	ForceSensor->SendForceRequest();	// リクエスト送信
	ForceSensor->WaitForceData();		// データ取得待機(ブロッキング動作)
	ForceSensor->Get6axisForce(Fx, Fy, Fz, Mx, My, Mz);	// [N],[Nm] 力覚センサ値の取得
}

//! @brief Z相クリアに関する設定をする関数
void Interface::SetZpulseClear(const bool flag){
	// インクリメンタルエンコーダのZ(I,C)相クリアの設定が必要な場合に記述する
	
}

