//! @file Interface.cc
//! @brief インターフェースクラス
//! @date 2019/07/28
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the BSD License.
// For details, see the License.txt file.

#include <memory>
#include <unistd.h>
#include "Interface.hh"
#include "ARCSeventlog.hh"
#include "ARCSassert.hh"
#include "ARCSprint.hh"
#include "PCI-3343A.hh"
#include "PCI-3180.hh"
#include "PCI-6205C.hh"
#include "Limiter.hh"

using namespace ARCS;

// インターフェースボードクラスの定義と実体
namespace {
	// ここにD/A，A/D，エンコーダボードクラス等々の定義と実体を記述する
	PCI3343A DAC(ConstParams::PCI3343A_ADDR, ConstParams::PCI3343A_ENA);
	PCI3180  ADC(ConstParams::PCI3180_ADDR, PCI3180::RANGE_B_5V, ConstParams::PCI3180_ENA);
	PCI6205C ENC(ConstParams::PCI6205C_ADDR1, ConstParams::PCI6205C_ADDR2,
				 ConstParams::PCI6205C_ADDR3, ConstParams::PCI6205C_ADDR4,
				 ConstParams::ACTUATOR_NUM, ConstParams::ENC_MULT4);
}

//! @brief コンストラクタ
Interface::Interface(){
	PassedLog();
	
	PassedLog();
}

//! @brief デストラクタ
Interface::~Interface(){
	SetZeroCurrent();	// 念のためのゼロ電流指令
	PassedLog();
}

//! @brief サーボON信号を送出する関数
void Interface::ServoON(void){
	// ここにサーボアンプへのサーボON信号の送出シーケンスを記述する
	
}

//! @brief サーボOFF信号を送出する関数
void Interface::ServoOFF(void){
	// ここにサーボアンプへのサーボOFF信号の送出シーケンスを記述する
	
}

//! @brief 電流指令をゼロに設定する関数
void Interface::SetZeroCurrent(void){
	// ここにゼロ電流指令とサーボアンプの関係を列記する
	DAC.SetVoltage(0,0,0,0);
}

//! @brief 位置応答を取得する関数
//! @param[out]	Position	位置応答
void Interface::GetPosition(
	std::array<double, ConstParams::ACTUATOR_MAX>& pos_e,
	std::array<double, ConstParams::ACTUATOR_MAX>& pos_m
){
	// ここにエンコーダとPosition配列との関係を列記する
	long count[PCI6205C::MAX_CH] = {0};
	ENC.GetCount(count);	// エンコーダカウント数を取得
	// モータ電気角の取得
	pos_e[0] = ConvElectAngle( count[0]);	// [rad] 1軸目
	pos_e[1] = ConvElectAngle(-count[1]);	// [rad] 2軸目 (エンコーダが逆に取り付けられているので注意)
	// モータ機械角の取得
	pos_m[0] = ConvMotorAngle( count[0]);	// [rad] 1軸目
	pos_m[1] = ConvMotorAngle(-count[1]);	// [rad] 2軸目 (エンコーダが逆に取り付けられているので注意)
}

//! @brief トルク応答と加速度応答を取得する関数
//! @param[out]	Torque	トルク応答
void Interface::GetTorqueAndAcceleration(
	std::array<double, ConstParams::ACTUATOR_MAX>& Torque,
	std::array<double, ConstParams::ACTUATOR_MAX>& Acceleration
){
	// ここにトルクセンサとTorque配列との関係を列記する
	double V1, V2, V3, V4;
	ADC.ConvStart();							// AD変換開始
	ADC.WaitBusy();								// AD変換が完了するまで待機(ブロッキング動作)
	ADC.GetVoltage(V1, V2, V3, V4);				// [V] 電圧値の取得
	Torque[0] = ConstParams::TRQSEN_TO_NM*V1;	// [Nm] 電圧からモータ1のトルク応答へ換算
	Torque[1] = ConstParams::TRQSEN_TO_NM*V2;	// [Nm] 電圧からモータ2のトルク応答へ換算
	Acceleration[0] = ConstParams::ACCSEN_TO_MPS2*V3;	// [m/s^2] 電圧からモータ1の負荷側接線加速度応答へ換算
}

//! @brief 電流応答を取得する関数
//! @param[out]	Current	電流応答
void Interface::GetCurrent(std::array<double, ConstParams::ACTUATOR_MAX>& Current){
	// ここに電流センサとCurrent配列との関係を列記する
	
}

//! @brief 電流指令を設定する関数
//! @param[in]	Current	電流指令
void Interface::SetCurrent(const std::array<double, ConstParams::ACTUATOR_MAX>& Current){
	// ここにCurrent配列とサーボアンプの関係を列記する
	double V1, V2, V3, V4;
	// 電流リミッタをかけてから電圧信号へ換算
	V1 = -ConstParams::SRV_A_TO_V*Limiter(Current[0], ConstParams::ACT_MAX_CURRENT[0]);	// U相電流指令 モータ1
	V2 = -ConstParams::SRV_A_TO_V*Limiter(Current[1], ConstParams::ACT_MAX_CURRENT[0]);	// V相電流指令 モータ1
	V3 = -ConstParams::SRV_A_TO_V*Limiter(Current[2], ConstParams::ACT_MAX_CURRENT[1]);	// U相電流指令 モータ2
	V4 = -ConstParams::SRV_A_TO_V*Limiter(Current[3], ConstParams::ACT_MAX_CURRENT[1]);	// V相電流指令 モータ2
	DAC.SetVoltage(V1, V2, V3, V4);	// DA変換電圧更新
}

//! @brief 6軸力覚センサ応答を取得する関数
//! @param[out]	Fx-Fy 各軸の並進力 [N]
//! @param[out]	Mx-My 各軸のトルク [Nm]
void Interface::Get6axisForce(double& Fx, double& Fy, double& Fz, double& Mx, double& My, double& Mz){
	// ここに6軸力覚センサと各変数との関係を列記する
	
}

//! @brief Z相クリアに関する設定をする関数
void Interface::SetZpulseClear(const bool flag){
	// インクリメンタルエンコーダのZ(I,C)相クリアの設定が必要な場合に記述する
	ENC.ZpulseClear(flag);
}

//! @brief モータ機械角 [rad] へ換算する関数
double Interface::ConvMotorAngle(long count){
	return ConstParams::ENC_TO_RADIAN*(double)count;
}

//! @brief モータ電気角 [rad] へ換算する関数 (-2π～+2πの値域制限あり)
double Interface::ConvElectAngle(long count){
	return ConstParams::ENC_TO_RADIAN*(double)(ConstParams::ENC_POLEPARE*( count % (ConstParams::ENC_MAX_COUNT/ConstParams::ENC_POLEPARE) ));
}

