//! @file GraphPlot.cc
//! @brief グラフプロットクラス
//!
//! グラフを描画するクラス
//!
//! @date 2019/07/29
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#ifndef GRAPHPLOT
#define GRAPHPLOT

#include <pthread.h>
#include "ConstParams.hh"
#include "CuiPlot.hh"

// 前方宣言
namespace ARCS {
	class ScreenParams;
	class FrameGraphics;
}

namespace ARCS {	// ARCS名前空間
	//! @brief グラフプロットクラス
	class GraphPlot {
		public:
			explicit GraphPlot(const std::string& Device);	//!< コンストラクタ
			GraphPlot(GraphPlot&& r);		//!< ムーブコンストラクタ
			~GraphPlot();					//!< デストラクタ
			void DrawAxes(void);			//!< プロット軸の描画
			void DrawLegends(void);			//!< 凡例の描画
			void DrawWaves(void);			//!< プロット波形の描画
			void ResetWaves(void);			//!< プロットをリセットする関数
			
			//! @brief プロット描画変数に値を設定する関数
			//! @param[in] u1...u2 インジケータの値
			template<typename T1, typename... T2>		// 可変長引数テンプレート
			static void SetVars(const T1& u1, const T2&... u2){
				// 再帰で順番に可変長引数を読み込んでいく
				if(VarsCount == 0){
					PlotNumBuf = (size_t)u1;	// 1個目の引数はプロット番号として格納
				}else{
					if(VarsCount < ConstParams::PLOT_VAR_MAX){
						VarsBuf.at(VarsCount - 1) = (double)u1;	// 有効な範囲内であれば指定値でバッファを埋める
					}
				}
				++VarsCount;	// 再帰カウンタをインクリメント
				SetVars(u2...);	// 自分自身を呼び出す(再帰)
			}
			static void SetVars(){
				// 再帰の最後に呼ばれる関数
				VarsCount = 0;	// すべての作業が終わったので，再帰カウンタを零に戻しておく
				pthread_mutex_lock(&PlotVarsMutex);
				PlotVars.at(PlotNumBuf) = VarsBuf;
				pthread_mutex_unlock(&PlotVarsMutex);
			}
		
		private:
			GraphPlot(const GraphPlot&) = delete;					//!< コピーコンストラクタ使用禁止
			const GraphPlot& operator=(const GraphPlot&) = delete;	//!< 代入演算子使用禁止
			
			FrameGraphics FG;	//!< フレームバッファ
			std::array<CuiPlot, ConstParams::PLOT_NUM> Plot;		//!< キュイプロットのクラス配列
			double t_offset;	//!< オフセット時間
			
			//! @brief プロット描画変数の実体
			static std::array<
				std::array<
					double, ConstParams::PLOT_VAR_MAX
				>, ConstParams::PLOT_MAX
			> PlotVars;
			static pthread_mutex_t PlotVarsMutex;	//!< プロット描画変数用のMutex
			
			static size_t PlotNumBuf;	//!< プロット描画番号バッファ
			static std::array<double, ConstParams::PLOT_VAR_MAX> VarsBuf;	//!< プロット描画変数バッファ
			static size_t VarsCount;	//!< 再帰カウンタ
	};
}

#endif

