//! @file ScreenParams.cc
//! @brief ARCS画面パラメータ格納クラス
//!        ARCS用画面に表示する各種パラメータを格納します。
//! @date 2019/02/24
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#include <tuple>
#include "ScreenParams.hh"
#include "ARCSeventlog.hh"

using namespace ARCS;

// 以下は静的メンバ変数の実体
// Mutexロック
pthread_mutex_t ScreenParams::TimeMutex  = PTHREAD_MUTEX_INITIALIZER;	//!< 時刻関連変数アクセス用TimeMutex
pthread_mutex_t ScreenParams::ActMutex   = PTHREAD_MUTEX_INITIALIZER;	//!< アクチュエータ関連変数アクセス用Mutex
pthread_mutex_t ScreenParams::IndicMutex = PTHREAD_MUTEX_INITIALIZER;	//!< 任意変数インジケータ関連変数アクセス用Mutex
pthread_mutex_t ScreenParams::OnsetMutex = PTHREAD_MUTEX_INITIALIZER;	//!< オンライン設定変数アクセス用Mutex

// リアルタイムスレッド関連の変数
double ScreenParams::Time = 0;													//!< [s] 時刻 (一番速いスレッド THREAD0 の時刻)
std::array<double, ConstParams::THREAD_MAX> ScreenParams::PeriodicTime = {0};	//!< [s] 計測された制御周期
std::array<double, ConstParams::THREAD_MAX> ScreenParams::ComputationTime = {0};//!< [s] 計測された消費時間
std::array<double, ConstParams::THREAD_MAX> ScreenParams::MaxTime = {0};		//!< [s] 計測された制御周期の最大値
std::array<double, ConstParams::THREAD_MAX> ScreenParams::MinTime = {0};		//!< [s] 計測された制御周期の最小値

// 状態フラグ関連の変数
bool ScreenParams::NetworkLink 	= false;	//!< ネットワークリンクフラグ
bool ScreenParams::Initializing = false;	//!< ロボット初期化フラグ

// アクチュエータ関連の変数
std::array<double, ConstParams::ACTUATOR_MAX> ScreenParams::CurrentRef  = {0};	//!< [A] アクチュエータの電流指令値
std::array<double, ConstParams::ACTUATOR_MAX> ScreenParams::PositionRes = {0};	//!< [m]/[rad] アクチュエータの位置応答値

// 任意変数インジケータ関連の変数
std::array<double, ConstParams::INDICVARS_MAX> ScreenParams::VarIndicator = {0};	//!< 任意変数表示値
std::array<double, ConstParams::INDICVARS_MAX> ScreenParams::VarIndicatorBuf = {0};	//!< 任意変数表示値バッファ
size_t ScreenParams::VarIndicCount = 0;		//!< 再帰カウンタ

// オンライン設定変数関連の変数
std::array<double, ConstParams::ONLINEVARS_MAX> ScreenParams::OnlineSetVar = {0};	//!< オンライン設定変数値
std::array<double, ConstParams::ONLINEVARS_MAX> ScreenParams::OnlineSetVarIni = {0};//!< オンライン設定変数の初期値
size_t ScreenParams::SetVarCount = 0;		//!< 再帰カウンタ
size_t ScreenParams::InitSetVarCount = 0;	//!< 再帰カウンタ

//! @brief 初期化関数
void ScreenParams::Init(void){
	// Mutexの初期化
	PassedLog();
	pthread_mutex_init(&TimeMutex, nullptr);
	pthread_mutex_init(&ActMutex, nullptr);
	pthread_mutex_init(&IndicMutex, nullptr);
	pthread_mutex_init(&OnsetMutex, nullptr);
}

//! @brief 時刻を取得する関数
//! @return	時刻
double ScreenParams::GetTime(void){
	double ret;
	pthread_mutex_lock(&TimeMutex);
	ret = Time;
	pthread_mutex_unlock(&TimeMutex);
	return ret;
}

//! @brief 時刻を設定する関数
//! @param[in]	t	時刻
void ScreenParams::SetTime(const double t){
	pthread_mutex_lock(&TimeMutex);
	Time = t;
	pthread_mutex_unlock(&TimeMutex);
}

//! @brief 実際の制御周期，消費時間，制御周期の最大値，最小値を返す関数
//! @param[in]	リアルタイムスレッド番号
//! @return 制御周期，消費時間，制御周期の最大値，最小値
std::tuple<double, double, double, double> ScreenParams::GetTimeVars(const unsigned int ThreadNum){
	double PT, CT, Max, Min;
	pthread_mutex_lock(&TimeMutex);
	PT  = PeriodicTime.at(ThreadNum);
	CT  = ComputationTime.at(ThreadNum);
	Max = MaxTime.at(ThreadNum);
	Min = MinTime.at(ThreadNum);
	pthread_mutex_unlock(&TimeMutex);
	return std::forward_as_tuple(PT, CT, Max, Min);
}

//! @brief 実際の制御周期，消費時間，制御周期の最大値，最小値の配列を返す関数
//! @param[out]	PT	制御周期の配列
//! @param[out]	CT	消費時間の配列
//! @param[out]	Max	制御周期の最大値の配列
//! @param[out]	Min	制御周期の最小値の配列
void ScreenParams::GetTimeVars(
	std::array<double, ConstParams::THREAD_MAX>& PT,
	std::array<double, ConstParams::THREAD_MAX>& CT,
	std::array<double, ConstParams::THREAD_MAX>& Max,
	std::array<double, ConstParams::THREAD_MAX>& Min
){
	pthread_mutex_lock(&TimeMutex);
	PT  = PeriodicTime;
	CT  = ComputationTime;
	Max = MaxTime;
	Min = MinTime;
	pthread_mutex_unlock(&TimeMutex);
}

//! @brief 実際の制御周期，消費時間，制御周期の最大値，最小値の配列を設定する関数
//! @param[in]	PT	制御周期の配列
//! @param[in]	CT	消費時間の配列
//! @param[in]	Max	制御周期の最大値の配列
//! @param[in]	Min	制御周期の最小値の配列
void ScreenParams::SetTimeVars(
	const std::array<double, ConstParams::THREAD_MAX>& PT,
	const std::array<double, ConstParams::THREAD_MAX>& CT,
	const std::array<double, ConstParams::THREAD_MAX>& Max,
	const std::array<double, ConstParams::THREAD_MAX>& Min
){
	pthread_mutex_lock(&TimeMutex);
	PeriodicTime = PT;
	ComputationTime = CT;
	MaxTime = Max;
	MinTime = Min;
	pthread_mutex_unlock(&TimeMutex);
}

//! @brief ネットワークリンクフラグを取得する関数
//! @return ネットワークリンクフラグ
bool ScreenParams::GetNetworkLink(void){
	return NetworkLink;
}

//! @brief ネットワークリンクフラグを設定する関数
//! @param[in] ネットワークリンクフラグ
void ScreenParams::SetNetworkLink(const bool LinkFlag){
	NetworkLink = LinkFlag;
}

//! @brief ロボット初期化フラグを取得する関数
//! @return ロボット初期化フラグ
bool ScreenParams::GetInitializing(void){
	return Initializing;
}

//! @brief ロボット初期化フラグを設定する関数
//! @param[in] ロボット初期化フラグ
void ScreenParams::SetInitializing(const bool InitFlag){
	Initializing = InitFlag;
}

//! @biref 電流と位置を取得する関数
//! @param[in]	アクチュエータ番号
//! @return	電流指令，位置応答
std::tuple<double, double> ScreenParams::GetCurrentAndPosition(const unsigned int ActNum){
	double Current, Position;
	pthread_mutex_lock(&ActMutex);
	Current  = CurrentRef.at(ActNum);
	Position = PositionRes.at(ActNum);
	pthread_mutex_unlock(&ActMutex);
	return std::forward_as_tuple(Current, Position);
}

//! @brief 電流と位置の配列を取得する関数
//! @param[in]	Current	電流指令の配列
//! @param[in]	Position	位置応答の配列
void ScreenParams::GetCurrentAndPosition(
	std::array<double, ConstParams::ACTUATOR_MAX>& Current,
	std::array<double, ConstParams::ACTUATOR_MAX>& Position
){
	pthread_mutex_lock(&ActMutex);
	Current  = CurrentRef;
	Position = PositionRes;
	pthread_mutex_unlock(&ActMutex);
}

//! @brief 電流と位置の配列を設定する関数
//! @param[in]	Current		電流指令
//! @param[in]	Position	位置応答
void ScreenParams::SetCurrentAndPosition(
	const std::array<double, ConstParams::ACTUATOR_MAX>& Current,
	const std::array<double, ConstParams::ACTUATOR_MAX>& Position
){
	pthread_mutex_lock(&ActMutex);
	CurrentRef  = Current;
	PositionRes = Position;
	pthread_mutex_unlock(&ActMutex);
}

//! @brief 任意変数インジケータの配列を返す関数
//! @param[out]	Vars	任意変数値の配列
void ScreenParams::GetVarIndicator(std::array<double, ConstParams::INDICVARS_MAX>& Vars){
	pthread_mutex_lock(&IndicMutex);
	Vars = VarIndicator;
	pthread_mutex_unlock(&IndicMutex);
}

//! @brief 任意変数インジケータの配列を設定する関数
//! @param[in]	任意変数値の配列
void ScreenParams::SetVarIndicator(const std::array<double, ConstParams::INDICVARS_MAX>& Vars){
	pthread_mutex_lock(&IndicMutex);
	VarIndicator = Vars;
	pthread_mutex_unlock(&IndicMutex);
}

//! @brief オンライン設定変数の配列を返す関数
//! @param[out]	オンライン設定変数値の配列
void ScreenParams::GetOnlineSetVar(std::array<double, ConstParams::ONLINEVARS_MAX>& Vars){
	pthread_mutex_lock(&OnsetMutex);
	Vars = OnlineSetVar;
	pthread_mutex_unlock(&OnsetMutex);
}

//! @brief オンライン設定変数の配列を設定する関数
//! @param[in]	オンライン設定変数値の配列
void ScreenParams::SetOnlineSetVar(const std::array<double, ConstParams::ONLINEVARS_MAX>& Vars){
	pthread_mutex_lock(&OnsetMutex);
	OnlineSetVar = Vars;
	pthread_mutex_unlock(&OnsetMutex);
}

//! @brief オンライン設定変数の配列を設定する関数
//! @param[in]	VarNum	変数番号
//! @param[in]	VarVal	オンライン設定変数値
void ScreenParams::SetOnlineSetVar(const unsigned int VarNum, const double VarVal){
	pthread_mutex_lock(&OnsetMutex);
	OnlineSetVar[VarNum] = VarVal;
	pthread_mutex_unlock(&OnsetMutex);
}


