//! @file ARCSeventlog.cc
//! @brief ARCS イベントログクラス
//!
//! ARCS用のイベントログクラス
//! 注意：リアルタイム空間ではこのライブラリが提供する関数は可能な限り使用しないこと。ジッタに影響するので。デバッグ時なら使用OK。
//!
//! @date 2019/03/29
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#include "ARCScommon.hh"
#include "ARCSeventlog.hh"
#include "ARCSscreen.hh"
#include "ConstParams.hh"

using namespace ARCS;

// 静的メンバ変数の実体
ARCSscreen* ARCSeventlog::ARCSscreenPtr = nullptr;	//!< ARCS画面ポインタ

//! @brief イベントログを初期化する関数
void ARCSeventlog::Init(void){
	// イベントログファイルの準備とヘッダの書き出し
	std::ofstream EventLogFile(ConstParams::EVENTLOG_NAME.c_str(), std::ios::out | std::ios::trunc);
	EventLogFile << "ARCS EVENT LOG FILE" << std::endl;
	EventLogFile << "DATE: " << ARCScommon::GetNowTime();
	EventLogFile << "CTRLNAME: " << ConstParams::CTRLNAME << std::endl;
	EventLogFile << "ARCS_REVISION: " << ConstParams::ARCS_REVISION << std::endl;
	EventLogFile << std::endl;
	PassedLog();
}

//! @brief ARCS画面ポインタの設定
//! @param[in]	ARCS画面へのポインタ
void ARCSeventlog::SetScreenPtr(ARCSscreen* ScrPtr){
	ARCSscreenPtr = ScrPtr;
}

//! @brief イベントログを残す関数 (任意メッセージ記録版)
//! @param[in] str 記録したいメッセージ
//! @param[in] file 記録したいファイル名
//! @param[in] line 記録したい行番号
void ARCSeventlog::EventLog_from_macro(const std::string& str, const std::string& file, const int line){
	WriteEventLog(str, file, line);		// イベントログをファイルに書き出す
	// イベントログデータを画面バッファに書き込む
	if(ARCSscreenPtr != nullptr){
		ARCSscreenPtr->WriteEventLogBuffer(file + " " + std::to_string(line) + ": " + str);
	}
}

//! @brief 変数用イベントログ
//! @param[in] u 表示する変数
//! @param[in] varname 変数名
void ARCSeventlog::EventLogVar_from_macro(const double u, const std::string& varname, const std::string& file, const int line){
	std::string str = varname + " = " + ARCScommon::DoubleToString(u,"%g");
	EventLog_from_macro(str, file, line);
}

//! @brief 通過確認用ログを残す関数（ファイルと行番号のみ記録版）
//! @param[in] str 記録したいメッセージ
//! @param[in] file 記録したいファイル名
//! @param[in] line 記録したい行番号
void ARCSeventlog::PassedLog_from_macro(const std::string& file, const int line){
	const std::string str = "PASSED";		// 任意メッセージは"PASSED"
	EventLog_from_macro(str, file, line);	// イベントログをファイルに書き出す＆画面表示
}

//! @brief イベントログをファイルに書き出す
//! @param[in] str ログに書き残したい文字列
//! @param[in] file ファイル名
//! @param[in] line 行番号
void ARCSeventlog::WriteEventLog(const std::string& str, const std::string& file, const int line){
	std::ofstream EventLogFile(ConstParams::EVENTLOG_NAME.c_str(), std::ios::out | std::ios::app);
	EventLogFile << file << " " << line << ": " << str <<std::endl;
}

