//! @file ARCSthread.cc
//! @brief ARCSリアルタイムスレッド管理クラス
//!
//! リアルタイムスレッドの生成、開始、停止、破棄などの管理をします。
//!
//! @date 2019/07/30
//! @author Yuki YOKOKURA
//
// Copyright (C) 2011-2019 Yuki YOKOKURA
// This program is free software;
// you can redistribute it and/or modify it under the terms of the FreeBSD License.
// For details, see the License.txt file.

#include <unistd.h>
#include "ARCSthread.hh"
#include "ARCScommon.hh"
#include "ARCSeventlog.hh"
#include "ARCSassert.hh"
#include "ConstParams.hh"
#include "ScreenParams.hh"

using namespace ARCS;

//! @brief コンストラクタ
ARCSthread::ARCSthread()
	: CtrlFuncs(),		// 制御用周期実行関数群の初期化
	  CtrlFuncObj(),	// 制御用周期実行関数の関数オブジェクトの初期化
	  RTthreads({
		SFthread(ConstParams::SAMPLING_TIME[0], ConstParams::CPUCORE_NUMBER[0]),	// リアルタイムスレッド1の初期化
		SFthread(ConstParams::SAMPLING_TIME[1], ConstParams::CPUCORE_NUMBER[1]),	// リアルタイムスレッド2の初期化
		SFthread(ConstParams::SAMPLING_TIME[2], ConstParams::CPUCORE_NUMBER[2]),	// リアルタイムスレッド3の初期化
	  }),
	  InfoGetThreadID()	// 情報取得スレッドの識別子の初期化
{
	PassedLog();
	CtrlFuncObj = CtrlFuncs.GetCtrlFuncObject();	// 制御用周期実行関数の関数オブジェクトを取得
	for(size_t i = 0; i < ConstParams::THREAD_NUM; ++i) RTthreads[i].SetRealtimeFunction(CtrlFuncObj[i]);	// それらをリアルタイムスレッドとして設定
	PassedLog();
}

//! @brief デストラクタ
ARCSthread::~ARCSthread(){
	PassedLog();
}

//! @ brief スレッドを開始する関数
void ARCSthread::Start(void){
	// 情報取得スレッド生成
	pthread_create(&InfoGetThreadID, NULL, (void*(*)(void*))InfoGetThread, this);			// 情報取得スレッド生成
	ARCScommon::SetCPUandPolicy(InfoGetThreadID, ConstParams::ARCS_UI_CPU_NUMBER, SCHED_RR);// CPUコアとポリシーの設定
	
	// 制御開始！
	PassedLog();
	CtrlFuncs.InitialProcess();		// 初期化モードの実行
	ARCSassert::SetRealtimeMode();	// ARCS用assertをリアルタイムモードに変更
	for(size_t i = 0; i < ConstParams::THREAD_NUM; ++i) RTthreads[i].Start();		// リアルタイムマルチスレッドの開始
	for(size_t i = 0; i < ConstParams::THREAD_NUM; ++i) RTthreads[i].WaitStart();	// 開始するまで待機
}

//! @brief スレッドを停止する関数
void ARCSthread::Stop(void){
	// 制御停止！
	if(ARCSassert::IsEmergency() == false){
		// 正常終了時は，
		ARCSassert::SetNonRealtimeMode();	// ARCS用assertを非リアルタイムモードに変更
		for(size_t i = 0; i < ConstParams::THREAD_NUM; ++i) RTthreads[i].Stop();	// リアルタイムマルチスレッドの停止
		for(size_t i = 0; i < ConstParams::THREAD_NUM; ++i) RTthreads[i].WaitStop();// リアルタイムマルチスレッドの終了待機
	}else{
		// 緊急停止時は，
		for(size_t i = 0; i < ConstParams::THREAD_NUM; ++i) RTthreads[i].Stop();	// リアルタイムマルチスレッドの停止
		// 緊急停止時は終了待機をしない
	}
	PassedLog();
	CtrlFuncs.ExitProcess();		// 終了処理モードの実行
	
	// 情報取得スレッド終了
	pthread_cancel(InfoGetThreadID);
}

//! @brief スレッドをリセットする関数
void ARCSthread::Reset(void){
	for(size_t i = 0; i < ConstParams::THREAD_NUM; ++i) RTthreads[i].Reset();	// リアルタイムマルチスレッドのリセット
}

//! @brief 測定データを保存する関数
void ARCSthread::SaveDataFiles(void){
	CtrlFuncs.SaveDataFiles();	// データを保存
}

//! @brief 情報取得スレッド
//! @param[in]	クラスメンバアクセス用ポインタ
void ARCSthread::InfoGetThread(ARCSthread* const p){
	double Time = 0;													// [s] 時刻 (一番速いスレッド RTthreads[0] の時刻)
	std::array<double, ConstParams::THREAD_MAX> PeriodicTime = {0};		// [s] 計測された制御周期
	std::array<double, ConstParams::THREAD_MAX> ComputationTime = {0};	// [s] 計測された消費時間
	std::array<double, ConstParams::THREAD_MAX> MaxTime = {0};			// [s] 計測された制御周期の最大値
	std::array<double, ConstParams::THREAD_MAX> MinTime = {0};			// [s] 計測された制御周期の最小値
	
	// スレッドがキャンセルされるまで無限ループ
	while(1){
		// リアルタイムスレッドから時刻情報を取得
		
		Time = p->RTthreads[0].GetTime();	// 時刻の取得
		
		for(size_t i = 0; i < ConstParams::THREAD_NUM; ++i){
			PeriodicTime[i]    = p->RTthreads[i].GetSmplTime();	// 制御周期の取得
			ComputationTime[i] = p->RTthreads[i].GetCompTime();	// 消費時間の取得
			MaxTime[i]         = p->RTthreads[i].GetMaxTime();	// 制御周期の最大値の取得
			MinTime[i]         = p->RTthreads[i].GetMinTime();	// 制御周期の最小値の取得
		}
		
		// ARCS画面パラメータに格納
		ScreenParams::SetTime(Time);
		ScreenParams::SetTimeVars(PeriodicTime, ComputationTime, MaxTime, MinTime);
		
		// 制御用変数値を更新
		p->CtrlFuncs.UpdateControlValue();			// 制御用周期実行関数群クラス内部で，ARCS画面パラメータに書き込む/から読み込む
		
		usleep(ConstParams::ARCS_UI_UPDATE_TIME);	// 指定時間だけ待機
	}
}

